/*
  Copyright (c) 2008 Sebastian Trueg <trueg@kde.org>

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License version 2 as published by the Free Software Foundation.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; see the file COPYING.LIB.  If not, write to
  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  Boston, MA 02110-1301, USA.
*/

#include "nepoid.h"
#include "pimo.h"

#include <QtGui/QTabBar>
#include <QtGui/QListView>
#include <QtGui/QVBoxLayout>
#include <QtGui/QGraphicsProxyWidget>
#include <QtGui/QGraphicsLinearLayout>
#include <QtGui/QGraphicsScene>

#include <QDebug>

#include <Nepomuk/Tag>
#include <Nepomuk/Tools>
#include <Nepomuk/Types/Class>
#include <Nepomuk/ResourceManager>

#include <Soprano/Vocabulary/NAO>

#include <KRun>
#include <KApplication>
#include <KCmdLineArgs>
#include <KAboutData>


#include "annotationresourcemodel.h"
#include "resource-visualization/resourcedelegate.h"

uint Nepomuk::qHash( const Nepomuk::Resource& res )
{
    return qHash( res.resourceUri().toString() );
}

namespace {
    enum Tabs {
        TagTab = 0,
        ContactsTab = 1,
        PlacesTab = 2
    };
}



// Nepoid::Nepoid( QObject* parent, const QVariantList& args )
//     : Plasma::Applet( parent, args )
// {
// }


Nepoid::Nepoid( QWidget* parent )
    : QWidget( parent )
{
    init();
}


void Nepoid::init()
{
    QWidget* frame = this;//new QWidget();
    QVBoxLayout* layout = new QVBoxLayout( frame );

    m_tabBar = new QTabBar( frame );
    m_listView = new QListView( frame );

    layout->addWidget( m_tabBar );
    layout->addWidget( m_listView );

    m_model = new AnnotationResourceModel( this );
    m_listView->setModel( m_model );

    m_listView->setDragDropMode( QAbstractItemView::DropOnly );
    m_listView->setAcceptDrops( true );
    m_listView->setDropIndicatorShown( true );

    m_listView->setItemDelegate( new Nepomuk::ResourceDelegate( m_listView ) );
    m_listView->setIconSize( QSize( 32, 32 ) );
    m_listView->setSpacing( 10 );

//     QGraphicsProxyWidget* proxy = new QGraphicsProxyWidget( this );
//     proxy->setWidget( frame );
//     QGraphicsLinearLayout* proxyLayout = new QGraphicsLinearLayout( this );
//     proxyLayout->addItem( proxy );
//     setLayout( proxyLayout );

    m_tabBar->addTab( i18n("Tags") );
    m_tabBar->addTab( i18n("Contacts") );
    m_tabBar->addTab( i18n("Places") );

    slotTabChanged( TagTab );

    connect( m_tabBar, SIGNAL(currentChanged(int)),
             this, SLOT(slotTabChanged(int)) );
    connect( m_listView, SIGNAL( activated( QModelIndex ) ),
             this, SLOT( slotItemActivated( QModelIndex ) ) );
}


Nepoid::~Nepoid()
{
}


namespace {
    QSet<Nepomuk::Resource> allResourcesOfTypeWithStupidInferencing( const QUrl& type ) {
        Nepomuk::Types::Class typeClass( type );
        QSet<Nepomuk::Resource> rl = Nepomuk::ResourceManager::instance()->allResourcesOfType( type ).toSet();
        foreach( const Nepomuk::Types::Class& c, typeClass.subClasses() ) {
            rl += allResourcesOfTypeWithStupidInferencing( c.uri() );
        }
        return rl;
    }
}

void Nepoid::slotTabChanged( int index )
{
    switch( index ) {
    case TagTab:
        m_model->setResources( Nepomuk::convertResourceList( Nepomuk::Tag::allTags() ) );
        break;

    case ContactsTab:
        m_model->setResources( allResourcesOfTypeWithStupidInferencing( QUrl("http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact") ).toList() );
        break;

    case PlacesTab:
        m_model->setResources( allResourcesOfTypeWithStupidInferencing( Nepomuk::Vocabulary::PIMO::Location() ).toList() );
        break;
    }
}


void Nepoid::slotItemActivated( const QModelIndex& index )
{
    QUrl uri = m_model->resourceForIndex( index );
    QString query;
    if ( m_tabBar->currentIndex() == TagTab ) {
        query = QString( "<%1>:<%2>" ).arg( QString::fromAscii( QUrl::toPercentEncoding( Soprano::Vocabulary::NAO::hasTag().toString() ) ),
                                            QString::fromAscii( QUrl::toPercentEncoding( uri.toString() ) ) );
    }
    else if ( m_tabBar->currentIndex() == ContactsTab ) {
        query = QString( "<%1>:<%2>" ).arg( QString::fromAscii( QUrl::toPercentEncoding( Nepomuk::Vocabulary::PIMO::isRelated().toString() ) ),
                                            QString::fromAscii( QUrl::toPercentEncoding( uri.toString() ) ) );
    }
    else {
        query = QString( "<%1>:<%2>" ).arg( QString::fromAscii( QUrl::toPercentEncoding( Nepomuk::Vocabulary::PIMO::hasLocation().toString() ) ),
                                            QString::fromAscii( QUrl::toPercentEncoding( uri.toString() ) ) );
    }

    KRun::runUrl( KUrl( QString( "nepomuksearch:/%1" ).arg( query ) ),
                  "inode/directory",
                  0 );
}


int main( int argc, char *argv[] )
{
    KAboutData aboutData( "nepoid", 0,
                          ki18n("Nepoid"),
                          "0.1",
                          ki18n("Nepomuk Akademy 2008 Demo"),
                          KAboutData::License_GPL,
                          ki18n("(c) 2008, Sebastian Trüg"),
                          KLocalizedString(),
                          "http://nepomuk.kde.org" );
    aboutData.addAuthor(ki18n("Sebastian Trüg"),ki18n("Maintainer"), "trueg@kde.org");
    aboutData.setProgramIconName( "nepomuk" );

    KCmdLineArgs::init( argc, argv, &aboutData );

    KCmdLineOptions options;
    KCmdLineArgs::addCmdLineOptions( options );

    KApplication app;

    Nepoid mainWin;
    mainWin.show();

    return app.exec();
}

#include "nepoid.moc"
