/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2008 Adrien Bustany <madcat@mymadcat.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include <KIconLoader>
#include <Nepomuk/Resource>
#include <Nepomuk/ResourceManager>
#include <soprano/soprano.h>
#include <soprano/asyncmodel.h>
#include <klocale.h>

#include "annotationplugin.h"
#include "annotationresource.h"
#include "annotationresult.h"

using namespace Nepomuk;

class AnnotationPlugin::Private : public QSharedData {
public:
	AnnotationProperty relatedTo;
	bool ready;
	Soprano::Util::AsyncModel* soprano;
	QString name;
	QString iconName;
};

AnnotationPlugin::AnnotationPlugin(QObject* parent) : QObject(parent), d(new Private)
{
	d->relatedTo = AnnotationProperty(this, Nepomuk::Types::Property(QUrl("http://www.semanticdesktop.org/ontologies/2007/08/15/nao#isRelated")), 0);
	d->relatedTo.setLabel(i18n("related to"));
	d->relatedTo.setDescription(i18n("the object is somehow linked to the selected file"));

	// Set up soprano
	d->soprano = new Soprano::Util::AsyncModel(Nepomuk::ResourceManager::instance()->mainModel());

	emit ready();
	d->ready = true;
}

AnnotationPlugin::~AnnotationPlugin()
{
	delete d->soprano;
}

bool AnnotationPlugin::isReady() const
{
	return d->ready;
}

QString AnnotationPlugin::name() const
{
	return "";
}

QString AnnotationPlugin::iconName() const
{
	return "unknown";
}

AnnotationResult* AnnotationPlugin::getResources(const QString& /*filter*/)
{
	AnnotationResult* r = new AnnotationResult(this);
	r->setReady();
	return r;
}

AnnotationResult* AnnotationPlugin::getResources(const QString& /*filter*/, const Types::Property& /*property*/, const Soprano::Node& /*object*/)
{
	AnnotationResult* r = new AnnotationResult(this);
	r->setReady();
	return r;
}

AnnotationResult* AnnotationPlugin::getResource(const QUrl& /*resourceUri*/)
{
	AnnotationResult* r = new AnnotationResult(this);
	r->setReady();
	return r;
}

AnnotationResult* AnnotationPlugin::getResourcesRelatedTo(const QUrl& /*resourceUri*/)
{
	AnnotationResult* r = new AnnotationResult(this);
	r->setReady();
	return r;
}

QList<Nepomuk::AnnotationProperty> AnnotationPlugin::getProperties(const QString& /*mimetype*/)
{
	QList<Nepomuk::AnnotationProperty> properties;

	// Add the relatedTo property with a relevance 0, as it is the most basic property
	properties.append(d->relatedTo);

	return properties;
}

QList<Nepomuk::AnnotationProperty> AnnotationPlugin::getProperties(const QUrl& /*rdfType*/)
{
	return getProperties(QString());
}

void AnnotationPlugin::addRelation(const Nepomuk::Types::Property& property, const QUrl& subject, const Soprano::Node& object)
{
	Soprano::Util::AsyncResult* result = d->soprano->addStatementAsync(subject, property.uri(), object); // DEBUG : bad, no error reporting
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SIGNAL(finished()));
	return;
}

void AnnotationPlugin::addRelation(const AnnotationProperty& property, const QUrl& subject, const Soprano::Node& object)
{
	addRelation(property.property(), subject, object);
}

AnnotationResult* AnnotationPlugin::createResult(AnnotationPlugin* parent)
{
	Q_ASSERT(parent != NULL);
	return new AnnotationResult(parent);
}

QString AnnotationPlugin::nodeToSparql(const Soprano::Node& node)
{
	if(node.isLiteral()) {
		return QString("\"%1\"^^<http://www.w3.org/2001/XMLSchema#string>").arg(node.literal().toString());
	}

	if(node.isResource()) {
		return QString("<%1>").arg(node.uri().toString());
	}

	return QString();
}

void AnnotationPlugin::enqueueCommand(AnnotationResult* r, const AnnotationPlugin::Command& c)
{
	m_queuedCommands[r] = c;
}

void AnnotationPlugin::executeQueuedCommands()
{
	foreach(AnnotationResult* r, m_queuedCommands.keys()) {
		Command command = m_queuedCommands.value(r);
		QHash<QString, QVariant> args = command.second;
		AnnotationResult* res;
		switch(command.first) {
			case command_getResources:
				res = getResources(args["filter"].toString());
				res->setParentResult(r);
				break;
			case command_getResourcesFiltered:
				res = getResources(args["filter"].toString(), args["property"].value<Nepomuk::Types::Property>(), args["object"].value<Soprano::Node>());
				res->setParentResult(r);
				break;	
			case command_getResource:
				res = getResource(args["resourceUri"].toUrl());
				res->setParentResult(r);
				break;
			case command_addRelationNepomukProperty:
				addRelation(args["property"].value<Nepomuk::Types::Property>(), args["subject"].value<QUrl>(), args["object"].value<Soprano::Node>());
				break;
			case command_addRelationAnnotationProperty:
				addRelation(args["property"].value<Nepomuk::AnnotationProperty>(), args["subject"].value<QUrl>(), args["object"].value<Soprano::Node>());
				break;
			default:
				break;
		}
	}
	m_queuedCommands.clear();
}

Q_DECLARE_METATYPE(Soprano::QueryResultIterator);
Q_DECLARE_METATYPE(Soprano::Node);
Q_DECLARE_METATYPE(Nepomuk::Types::Property);
Q_DECLARE_METATYPE(Nepomuk::AnnotationProperty);

////////// Result class //////////
#include "annotationplugin.moc"
