/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2008 Adrien Bustany <madcat@mymadcat.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef DATAPROVIDER_H
#define DATAPROVIDER_H

#include <QtCore>
#include <nepomuk/nepomuk_export.h>
#include <Nepomuk/Types/Property>
#include "annotationproperty.h"

namespace Soprano {
	class Node;
}

namespace Nepomuk {

class AnnotationResource;
class AnnotationResult;

/**
 * \class AnnotationPlugin annotationplugin.h AnnotationPlugin
 *
 * \brief A generic implementation of a Nepomuk AnnotationPlugin
 *
 * AnnotationPlugins are designed to provide an easy to use interface to data stored into Nepomuk.
 * The interface is all asynchronous, if you ever need to use AnnotationPlugins in a threaded environment,
 * use the SyncAnnotationPlugin class.
 *
 * All the get* functions return an AnnotationResult object.
 *
 * \sa SyncAnnotationPlugin
 * \sa AnnotationResult
 *
 * \author Adrien Bustany <madcat@mymadcat.com>
 */
class NEPOMUK_EXPORT AnnotationPlugin : public QObject
{
	Q_OBJECT

public:
	AnnotationPlugin(QObject* parent);
	virtual ~AnnotationPlugin();

	/**
	 * Returns true if the plugin is ready, false else.
	 * This function is useful when you get several times an instance of the plugin. As the factory will only create one
	 * instance of each plugin, the ready() signal will only be emitted once.
	 * If you execute commands on a non-ready plugin, they'll be queued and executed when the plugin is ready.
	 */
	virtual bool isReady() const;

	/**
	 * Returns the name of the plugin
	 */
	virtual QString name() const;

	/**
	 * Returns an icon name usable for example with KIcon
	 */
	virtual QString iconName() const;

signals:
	/**
	 * Emitted when addRelation finishes
	 */
	void finished();

	/**
	 * Emitted when the plugin is ready to use. If any command is issued before the
	 * plugin is ready, it'll be queued and executed automatically when possible.
	 */
	void ready();

public Q_SLOTS:
	/**
	 * Retrieves all the resources related with the \a filter keyword
	 */
	virtual AnnotationResult* getResources(const QString& filter);

	/**
	 * Retrieves all the resources related with the \a filter keyword, and having the given \a property
	 */
	virtual AnnotationResult* getResources(const QString& filter, const Types::Property& property, const Soprano::Node& object);

	/**
	 * Retrieves the resource with the given \a uri
	 */
	virtual AnnotationResult* getResource(const QUrl& resourceUri);

	/**
	 * Retrieves all the resources linked to \p resourceUri . This function uses nao:isRelated. This relation being a
	 * symmetric property, we also do a reverse query.
	 */
	virtual AnnotationResult* getResourcesRelatedTo(const QUrl& resourceUri);

	/**
	 * Retrieves all the relevant relations for the given mimetype as a list of <Property, Relevance> tuples
	 */
	virtual QList<AnnotationProperty> getProperties(const QString& mimetype);

	/**
	 * Retrieves all the relevant relations for the given RDF type as a list of <Property, Relevance> tuples
	 */
	virtual QList<AnnotationProperty> getProperties(const QUrl& rdfType);

	/**
	 * Effectively associates the file at the given url with the relation identified by the uri
	 */
	virtual void addRelation(const Types::Property& property, const QUrl& subject, const Soprano::Node& object);
	virtual void addRelation(const AnnotationProperty& property, const QUrl& subject, const Soprano::Node& object);

private:
	class Private;
	QSharedDataPointer<Private> d;

protected:
	/**
	 * Creates a new AnnotationResult with the given \p parent
	 *
	 * This is used by subclasses to create a new AnnotationResult objects, since the constructor
	 * is private.
	 */
	AnnotationResult* createResult(AnnotationPlugin* parent);

	/**
	 * Converts a soprano node to a QString usable in a sparql query
	 */
	QString nodeToSparql(const Soprano::Node& node);

	/**
	 * Only for internal use : used to queue commands when the plugin is not ready
	 * Note : getProperties cannot be enqueud since it does not return void.
	 */
	enum CommandName {
		command_getResources,
		command_getResourcesFiltered,
		command_getResource,
		command_addRelationNepomukProperty,
		command_addRelationAnnotationProperty
	};
	typedef QPair<AnnotationPlugin::CommandName, QHash<QString, QVariant> > Command;

	QMap<AnnotationResult*, AnnotationPlugin::Command> m_queuedCommands;

	/**
	 * Used by the plugin to memorize the commands to execute once it'll be ready
	 */
	void enqueueCommand(AnnotationResult* r, const AnnotationPlugin::Command &c);

	/**
	 * Used by the plugin to execute all the enqueued commands once it's ready
	 */
	void executeQueuedCommands();

};

} // End namespace Nepomuk

#endif
