#ifndef ANNOTATIONRESULT_H
#define ANNOTATIONRESULT_H

#include <QtCore>

#include "annotationresource.h"


namespace Nepomuk {

class AnnotationPlugin;

/**
 * \class AnnotationResult annotationresult.h AnnotationResult
 * \author Adrien Bustany <madcat@mymadcat.com>
 *
 * \brief Handle to get the results from a plugin query.
 *
 * Typical use :
 * \code
 * Nepomuk::AnnotationResult* res = plugin->getResources("filter");
 * // We will get the results when they're ready with getResources()
 * connect(res, SIGNAL(ready()), this, SLOT(result_cb()));
 * res->emitStatus(); // Emits ready if ever the plugin was created ready
 * \endcode
 *
 * The results have for parent the plugin which created them, which means they will be freed one day. Anyway
 * it's highly advised to delete them when you don't need them anymore.
 */
class KDE_EXPORT AnnotationResult : public QObject
{
	Q_OBJECT

	public:
		~AnnotationResult();

		/**
		 * Returns true after the ready() signal has been emitted
		 */
		bool isReady() const;

		/**
		 * Emits ready() is the plugin is ready. This function is useful in the case where the result is
		 * created and directly ready (so you can't catch the first ready signal), eg :
		 * \code
		 * AnnotationResult* readyNowResult = plugin->getResources("foo");
		 * connect(readyNowResult, SIGNAL(ready()), receiver, SLOT(result_cb()));
		 * readyNowResult->emitStatus();
		 * \endcode
		 */
		void emitStatus();

		AnnotationPlugin* parentPlugin();

		/**
		 * Returns the results of this query. If not ready, return an empty list.
		 */
		const QList<AnnotationResource>& resources();


		void setResources(const QList<AnnotationResource>& resources);

		void setReady();

		// Bit hackish way to handle the queued commands... If the Result has a parent, it will proxy
		// all the results to the parent, and store nothing itself
		void setParentResult(AnnotationResult* parent);

	Q_SIGNALS:
		/**
		 * Emitted when the results of the query are ready.
		 */
		void ready();

	private:
		AnnotationResult(AnnotationPlugin* parent);
		friend class AnnotationPlugin;

		class Private;
		Private* d;
};

} // End namespace Nepomuk

#endif
