/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2008 Adrien Bustany <madcat@mymadcat.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "nco.h"
#include "nepomukproxy.h"

#include "annotationresource.h"
#include "annotationresult.h"

#include <kiconloader.h>
#include <KLocalizedString>
#include <Nepomuk/Resource>
#include <Nepomuk/ResourceManager>
#include <soprano/asyncmodel.h>

#define NCO "http://www.semanticdesktop.org/ontologies/2007/03/22/nco#"
#define NFO "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#"
#define NIR "http://www.semanticdesktop.org/ontologies/2008/07/04/nir#"

using namespace Nepomuk;

NcoAnnotationPlugin::NcoAnnotationPlugin(QObject* parent, const QList<QVariant>&) : AnnotationPlugin(parent), m_ready(false)
{
	m_nepomuk = NepomukProxy::getInstance();
	connect(m_nepomuk, SIGNAL(contacts(const QList<Soprano::BindingSet>&)), this, SLOT(contacts_cb(const QList<Soprano::BindingSet>&)));
	connect(m_nepomuk, SIGNAL(regionAdded()), this, SIGNAL(finished()));
	m_nepomuk->getContacts();

	m_regionDepicts = AnnotationProperty(this, QUrl(NIR "regionDepicts"), 10);
	m_regionDepicts.setLabel(i18n("is on picture"));
	m_regionDepicts.setDescription(i18n("the person is on the selected picture"));

	m_soprano = new Soprano::Util::AsyncModel(Nepomuk::ResourceManager::instance()->mainModel());
}

NcoAnnotationPlugin::~NcoAnnotationPlugin()
{
	delete m_soprano;
}

bool NcoAnnotationPlugin::isReady() const
{
	return m_ready;
}

QString NcoAnnotationPlugin::name() const
{
	return i18n("Contacts");
}

QString NcoAnnotationPlugin::iconName() const
{
	return "user-identity";
}

AnnotationResult* NcoAnnotationPlugin::getResources(const QString& filter)
{
	AnnotationResult* result = AnnotationPlugin::createResult(this);

	// This function needs the cached data, enqueue the command if we're not ready yet
	if(!isReady()) {
		QHash<QString, QVariant> args;
		args["filter"] = filter;
		enqueueCommand(result, qMakePair(AnnotationPlugin::command_getResources, args));
		return result;
	}
		
	QList<Nepomuk::AnnotationResource> res;
	foreach(const AnnotationResource& r, m_resources) {
		if(r.label().contains(filter, Qt::CaseInsensitive) || r.description().contains(filter, Qt::CaseInsensitive))
			res.append(r);
	}

	result->setResources(res);
	result->setReady();
	return result;
}

AnnotationResult* NcoAnnotationPlugin::getResources(const QString& filter, const Types::Property& property, const Soprano::Node& object)
{
	AnnotationResult* result = AnnotationPlugin::createResult(this);

	// Duplicates the Nepomuk proxy a bit, but we don't have the choice :-s
	QString query = QString("select ?uid ?name ?gid ?gname where {"
							"?uid a <" NCO "PersonContact> ."
							"?uid <" NCO "fullname> ?name ."
							"?uid <%1> %2 ."
							"optional { ?uid <" NCO "belongsToGroup> ?gid . "
							"?gid a <" NCO "ContactGroup> ."
							"?gid <" NCO "contactGroupName> ?gname }}").arg(property.uri().toString(), nodeToSparql(object));
	qDebug() << query;
	Soprano::Util::AsyncResult* sopranoResult = m_soprano->executeQueryAsync(query, Soprano::Query::QueryLanguageSparql);
	connect(sopranoResult, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(contacts_with_property_cb(Soprano::Util::AsyncResult*)));
	m_queryFilters[sopranoResult] = filter;

	m_resultsTable[sopranoResult] = result;

	return result;
}

AnnotationResult* NcoAnnotationPlugin::getResource(const QUrl& resourceUri)
{
	AnnotationResult* result = AnnotationPlugin::createResult(this);

	// This function needs the cached data, enqueue the command if we're not ready yet
	if(!isReady()) {
		QHash<QString, QVariant> args;
		args["resourceUri"] = resourceUri;
		enqueueCommand(result, qMakePair(AnnotationPlugin::command_getResource, args));
		return result;
	}

	foreach(const AnnotationResource& r, m_resources) {
		if(r.resource().resourceUri() == resourceUri) {
			result->setResources(QList<Nepomuk::AnnotationResource>() << r);
			result->setReady();
			return result;
		}
	}
	
	result->setReady();
	return result;
}

AnnotationResult* NcoAnnotationPlugin::getResourcesRelatedTo(const QUrl& resourceUri)
{
	AnnotationResult* result = AnnotationPlugin::createResult(this);

	QString query = QString("select ?uid ?name ?gid ?gname where {"
							"?uid a <" NCO "PersonContact> ."
							"?uid <" NCO "fullname> ?name ."
							"?property <%1> <%2> ."
							"{{?uid ?property <%3>} UNION {<%3> ?property ?uid}} ."
							"optional { ?uid <" NCO "belongsToGroup> ?gid . "
							"?gid a <" NCO "ContactGroup> ."
							"?gid <" NCO "contactGroupName> ?gname }}").
							arg(Soprano::Vocabulary::RDFS::subPropertyOf().toString(),
								Soprano::Vocabulary::NAO::annotation().toString(), resourceUri.toString());
	qDebug() << query;
	Soprano::Util::AsyncResult* sopranoResult = m_soprano->executeQueryAsync(query, Soprano::Query::QueryLanguageSparql);
	connect(sopranoResult, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(contacts_relatedTo_cb(Soprano::Util::AsyncResult*)));

	m_resultsTable[sopranoResult] = result;

	return result;
}

QList<Nepomuk::AnnotationProperty> NcoAnnotationPlugin::getProperties(const QString& mimetype)
{
	QList<Nepomuk::AnnotationProperty> properties;

	// Add "is on picture" from NIR if the file is an image
	if(mimetype.startsWith("image/")) {
		properties.append(m_regionDepicts);
	}

	// Add the default relatedTo relation
	properties += AnnotationPlugin::getProperties(mimetype);

	return properties;
}

QList<Nepomuk::AnnotationProperty> NcoAnnotationPlugin::getProperties(const QUrl& rdfType)
{
	if(rdfType == QUrl(NFO "Image")) {
		return getProperties("image/");
	}
	return AnnotationPlugin::getProperties(rdfType);
}

void NcoAnnotationPlugin::addRelation(const Nepomuk::Types::Property& property, const QUrl& subject, const Soprano::Node& object)
{
	if(property == m_regionDepicts.property()) {
		// Here we create an "invalid" region, null rectangle = indefinite region
		m_nepomuk->addRegion(subject.path(), QRectF(0, 0, 0, 0), object.uri().toString());
		return;
	}
}

void NcoAnnotationPlugin::addRelation(const AnnotationProperty& property, const QUrl& subject, const Soprano::Node& object)
{
	if(property.property() == m_regionDepicts.property())
		addRelation(property.property(), subject, object);
}

void NcoAnnotationPlugin::contacts_cb(const QList<Soprano::BindingSet>& results)
{
	m_resources.clear();
	foreach(const Soprano::BindingSet& set, results) {
		// Nepomuk will return several statements for one contact if it is in several groups
		// To prevent having the same contacts several times in the list, we search for an existing occurence before adding it
		int index = indexFromUri(m_resources, set["uid"].uri());
		if(index == -1) {
			AnnotationResource i(this, set["uid"].uri(), set["name"].literal().toString(), set["gname"].literal().toString(), QString("user-identity"));
			m_resources.append(i);
		} else {
			m_resources[index].setDescription(m_resources.at(index).description() + QString(", %1").arg(set["gname"].literal().toString()));
		}
	}
	m_ready = true;
	executeQueuedCommands();
	emit ready();
}

void NcoAnnotationPlugin::contacts_with_property_cb(Soprano::Util::AsyncResult* result)
{
	QList<AnnotationResource> res;
	foreach(const Soprano::BindingSet& set, result->value().value<Soprano::QueryResultIterator>().allBindings()) {
		// Nepomuk will return several statements for one contact if it is in several groups
		// To prevent having the same contacts several times in the list, we search for an existing occurence before adding it
		int index = indexFromUri(res, set["uid"].uri());
		if(index == -1) {
			AnnotationResource i(this, set["uid"].uri(), set["name"].literal().toString(), set["gname"].literal().toString(), QString("user-identity"));
			res.append(i);
		} else {
			res[index].setDescription(res.at(index).description() + QString(", %1").arg(set["gname"].literal().toString()));
		}
	}

	QString filter = m_queryFilters[result];
	foreach(const AnnotationResource& r, res) {
		if(!r.label().contains(filter, Qt::CaseInsensitive) && !r.description().contains(filter, Qt::CaseInsensitive))
			res.removeOne(r);
	}
	m_queryFilters.remove(result);

	AnnotationResult* r = m_resultsTable[result];
	r->setResources(res);
	r->setReady();
	m_resultsTable.remove(result);
}

void NcoAnnotationPlugin::contacts_relatedTo_cb(Soprano::Util::AsyncResult* result)
{
	QList<AnnotationResource> res;
	foreach(const Soprano::BindingSet& set, result->value().value<Soprano::QueryResultIterator>().allBindings()) {
		// Nepomuk will return several statements for one contact if it is in several groups
		// To prevent having the same contacts several times in the list, we search for an existing occurence before adding it
		int index = indexFromUri(res, set["uid"].uri());
		if(index == -1) {
			AnnotationResource i(this, set["uid"].uri(), set["name"].literal().toString(), set["gname"].literal().toString(), QString("user-identity"));
			res.append(i);
		} else {
			res[index].setDescription(res.at(index).description() + QString(", %1").arg(set["gname"].literal().toString()));
		}
	}

	AnnotationResult* r = m_resultsTable[result];
	r->setResources(res);
	r->setReady();
	m_resultsTable.remove(result);
}

int NcoAnnotationPlugin::indexFromUri(const QList<AnnotationResource>& list, const QUrl& uri)
{
	for(int i = 0 ; i < list.size() ; ++i) {
		if(list.at(i).resource().resourceUri() == uri)
			return i;
	}
	return -1;
}

Q_DECLARE_METATYPE(Soprano::QueryResultIterator);

#include "nco.moc"
