/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2008 Adrien Bustany <madcat@mymadcat.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "webpage.h"

#include <KIconLoader>
#include <Nepomuk/Resource>
#include <Nepomuk/ResourceManager>
#include <soprano/soprano.h>
#include <soprano/asyncmodel.h>
#include <klocale.h>

#include "annotationresult.h"

#define PIMO_WEBPAGE_TYPE "http://www.semanticdesktop.org/ontologies/2007/11/01/pimo#Document"
#define PIMO_GROUNDINGOCCURRENCE "http://www.semanticdesktop.org/ontologies/2007/11/01/pimo#groundingOccurrence"
#define NFO_WEBSITE_TYPE "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#Website"

using namespace Nepomuk;

WebpageAnnotationPlugin::WebpageAnnotationPlugin(QObject* parent, const QList<QVariant>&) : AnnotationPlugin(parent), m_ready(false)
{
	m_soprano = new Soprano::Util::AsyncModel(Nepomuk::ResourceManager::instance()->mainModel());
	fetchPages();
}

WebpageAnnotationPlugin::~WebpageAnnotationPlugin()
{
	delete m_soprano;
}

bool WebpageAnnotationPlugin::isReady() const
{
	return m_ready;
}

QString WebpageAnnotationPlugin::name() const
{
	return i18n("Documents");
}

QString WebpageAnnotationPlugin::iconName() const
{
	return "browser";
}

AnnotationResult* WebpageAnnotationPlugin::getResources(const QString& filter)
{
	AnnotationResult* result = AnnotationPlugin::createResult(this);

	// This function needs the cached data, enqueue the command if we're not ready yet
	if(!isReady()) {
		QHash<QString, QVariant> args;
		args["filter"] = filter;
		enqueueCommand(result, qMakePair(AnnotationPlugin::command_getResources, args));
		return result;
	}

	QList<AnnotationResource> res;
	foreach(const AnnotationResource& r, m_resources) {
		if(r.label().contains(filter, Qt::CaseInsensitive) || r.description().contains(filter, Qt::CaseInsensitive))
			res.append(r);
	}

	result->setResources(res);
	result->setReady();
	return result;
}

AnnotationResult* WebpageAnnotationPlugin::getResources(const QString& filter, const Types::Property& property, const Soprano::Node& object)
{
	AnnotationResult* result = AnnotationPlugin::createResult(this);

	QString query = QString("select ?pimoid ?nfoid ?title where {"
							"?nfoid a <" NFO_WEBSITE_TYPE "> ."
							"?pimoid a <" PIMO_WEBPAGE_TYPE "> ."
							"?pimoid <" PIMO_GROUNDINGOCCURRENCE "> ?nfoid ."
							"?pimoid <%1> %2 ."
							"optional { ?pimoid <%3> ?title } }")
							.arg(property.uri().toString(), nodeToSparql(object), Soprano::Vocabulary::NAO::prefLabel().toString());
	Soprano::Util::AsyncResult* sopranoResult = m_soprano->executeQueryAsync(query, Soprano::Query::QueryLanguageSparql);
	connect(sopranoResult, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(result_with_property_cb(Soprano::Util::AsyncResult*)));
	m_queryFilters[sopranoResult] = filter;
	m_resultsTable[sopranoResult] = result;

	return result;
}

AnnotationResult* WebpageAnnotationPlugin::getResource(const QUrl& resourceUri)
{
	AnnotationResult* result = AnnotationPlugin::createResult(this);

	// This function needs the cached data, enqueue the command if we're not ready yet
	if(!isReady()) {
		QHash<QString, QVariant> args;
		args["resourceUri"] = resourceUri;
		enqueueCommand(result, qMakePair(AnnotationPlugin::command_getResource, args));
		return result;
	}

	foreach(const AnnotationResource& r, m_resources) {
		if(r.resource().resourceUri() == resourceUri) {
			result->setResources(QList<Nepomuk::AnnotationResource>() << r);
			result->setReady();
			return result;
		}
	}
	result->setReady();
	return result;
}

AnnotationResult* WebpageAnnotationPlugin::getResourcesRelatedTo(const QUrl& resourceUri)
{
	AnnotationResult* result = AnnotationPlugin::createResult(this);

	QString query = QString("select ?pimoid ?nfoid ?title where {"
							"?nfoid a <" NFO_WEBSITE_TYPE "> ."
							"?pimoid a <" PIMO_WEBPAGE_TYPE "> ."
							"?pimoid <" PIMO_GROUNDINGOCCURRENCE "> ?nfoid ."
							"?property <%1> <%2> ."
							"{{?pimoid ?property <%3>} UNION {<%3> ?property ?pimoid}} ."
							"optional { ?pimoid <%4> ?title } }")
							.arg(Soprano::Vocabulary::RDFS::subPropertyOf().toString(),
								Soprano::Vocabulary::NAO::annotation().toString(),
								resourceUri.toString(),
								Soprano::Vocabulary::NAO::prefLabel().toString());
	qDebug() << query;
	Soprano::Util::AsyncResult* sopranoResult = m_soprano->executeQueryAsync(query, Soprano::Query::QueryLanguageSparql);
	connect(sopranoResult, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(result_relatedTo_cb(Soprano::Util::AsyncResult*)));

	m_resultsTable[sopranoResult] = result;

	return result;
}

QList<AnnotationProperty> WebpageAnnotationPlugin::getProperties(const QString& /*mimetype*/)
{
	return AnnotationPlugin::getProperties(QString());
}

QList<AnnotationProperty> WebpageAnnotationPlugin::getProperties(const QUrl& /*rdfType*/)
{
	return AnnotationPlugin::getProperties(QString());
}

void WebpageAnnotationPlugin::fetchPages()
{
	QString query = QString("select ?pimoid ?nfoid ?title where {"
							"?nfoid a <" NFO_WEBSITE_TYPE "> ."
							"?pimoid a <" PIMO_WEBPAGE_TYPE "> ."
							"?pimoid <" PIMO_GROUNDINGOCCURRENCE "> ?nfoid ."
							"optional { ?pimoid <%1> ?title } }").arg(Soprano::Vocabulary::NAO::prefLabel().toString());
	Soprano::Util::AsyncResult* result = m_soprano->executeQueryAsync(query, Soprano::Query::QueryLanguageSparql);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(result_cb(Soprano::Util::AsyncResult*)));
}

void WebpageAnnotationPlugin::result_cb(Soprano::Util::AsyncResult* results)
{
	m_resources.clear();

	foreach(const Soprano::BindingSet& set, results->value().value<Soprano::QueryResultIterator>().allBindings()) {
		m_resources.append(AnnotationResource(this, Resource(set["pimoid"].uri()),
					set["title"].isEmpty() ? set["nfoid"].uri().toString() : set["title"].literal().toString(),
					set["title"].isEmpty() ? QString() : set["nfoid"].uri().toString(),
					QString("text-html")));
	}
	
	m_ready = true;
	executeQueuedCommands();
	emit ready();
}

void WebpageAnnotationPlugin::result_with_property_cb(Soprano::Util::AsyncResult* results)
{
	QList<AnnotationResource> res;

	foreach(const Soprano::BindingSet& set, results->value().value<Soprano::QueryResultIterator>().allBindings()) {
		res.append(AnnotationResource(this, Resource(set["pimoid"].uri()),
					set["title"].isEmpty() ? set["nfoid"].uri().toString() : set["title"].literal().toString(),
					set["title"].isEmpty() ? QString() : set["nfoid"].uri().toString(),
					QString("text-html")));
	}

	QString filter = m_queryFilters[results];
	foreach(const AnnotationResource& r, res) {
		if(!r.label().contains(filter, Qt::CaseInsensitive) && !r.description().contains(filter, Qt::CaseInsensitive))
			res.removeOne(r);
	}
	m_queryFilters.remove(results);
	
	AnnotationResult* result = m_resultsTable[results];
	result->setResources(res);
	result->setReady();
	m_resultsTable.remove(results);
}

void WebpageAnnotationPlugin::result_relatedTo_cb(Soprano::Util::AsyncResult* results)
{
	QList<AnnotationResource> res;

	foreach(const Soprano::BindingSet& set, results->value().value<Soprano::QueryResultIterator>().allBindings()) {
		res.append(AnnotationResource(this, Resource(set["pimoid"].uri()),
					set["title"].isEmpty() ? set["nfoid"].uri().toString() : set["title"].literal().toString(),
					set["title"].isEmpty() ? QString() : set["nfoid"].uri().toString(),
					QString("text-html")));
	}
	
	AnnotationResult* result = m_resultsTable[results];
	result->setResources(res);
	result->setReady();
	m_resultsTable.remove(results);
}


Q_DECLARE_METATYPE(Soprano::QueryResultIterator);

#include "webpage.moc"

