/*
 *
 * $Id: sourceheader 511311 2006-02-19 14:51:05Z trueg $
 *
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2007-2008 Sebastian Trueg <trueg@kde.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#include "kio_tags.h"

#include <nepomuk/tag.h>
#include <nepomuk/variant.h>
#include <nepomuk/resourcemanager.h>

#include <KUrl>
#include <KAboutData>
#include <KApplication>
#include <KCmdLineArgs>
#include <kio/global.h>
#include <klocale.h>
#include <kio/job.h>
#include <KUser>
#include <KDebug>

#include <QtCore/QEventLoop>

#include <Soprano/Vocabulary/Xesam>
#include <Soprano/Vocabulary/NAO>


using namespace KIO;


namespace {
    KIO::UDSEntry createUDSEntryForTag( const Nepomuk::Tag& tag ) {
        // create a virtual folder stat thingi
        KIO::UDSEntry uds;
        uds.insert( KIO::UDSEntry::UDS_NAME, tag.genericLabel() );
        uds.insert( KIO::UDSEntry::UDS_DISPLAY_NAME, i18n( "Things tagged with '%1'", uds.stringValue( KIO::UDSEntry::UDS_NAME ) ) );
        uds.insert( KIO::UDSEntry::UDS_CREATION_TIME, tag.property( Soprano::Vocabulary::NAO::created() ).toDateTime().toTime_t() );
        uds.insert( KIO::UDSEntry::UDS_ACCESS, 0700 );
        uds.insert( KIO::UDSEntry::UDS_USER, KUser().loginName() );
        uds.insert( KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR );
        uds.insert( KIO::UDSEntry::UDS_MIME_TYPE, QString::fromLatin1( "inode/directory" ) );
        QString icon = tag.genericIcon();
        if ( !icon.isEmpty() )
            uds.insert( KIO::UDSEntry::UDS_ICON_NAME, icon );
        return uds;
    }
}



Nepomuk::TagsProtocol::TagsProtocol( const QByteArray& poolSocket, const QByteArray& appSocket )
    : KIO::ForwardingSlaveBase( "tags", poolSocket, appSocket )
{
    kDebug();
    // FIXME: connect to main model and listen for changes
}


Nepomuk::TagsProtocol::~TagsProtocol()
{
    kDebug();
}


void Nepomuk::TagsProtocol::listDir( const KUrl& url )
{
    kDebug();

    // FIXME: always list all tags except the ones already in the path. These tags (subfolders)
    // then allow to browse "Things tagged with 'A', 'B', and 'C'" and such

    QString path = url.path();
    if ( path.isEmpty() || path == "/" ) {
        listTags();
    }
    else {
        QString tag;
        if ( splitUrl( url, tag, path ) ) {
            if ( path.isEmpty() && !tag.isEmpty() ) {
                listTag( tag );
            }
            else {
                ForwardingSlaveBase::listDir( url );
            }
        }
    }
}


void Nepomuk::TagsProtocol::mkdir( const KUrl &url, int permissions )
{
    kDebug();

    QString tag, path;
    if ( splitUrl( url, tag, path ) ) {
        if ( path.isEmpty() && !tag.isEmpty() ) {
            Tag( tag ).setLabel( tag );
            finished();
        }
        else {
            ForwardingSlaveBase::mkdir( url, permissions );
        }
    }
}


void Nepomuk::TagsProtocol::get( const KUrl& url )
{
    kDebug();
    ForwardingSlaveBase::get( url );
}


void Nepomuk::TagsProtocol::put( const KUrl& url, int permissions, KIO::JobFlags flags )
{
    kDebug();
    Q_UNUSED( url );
    Q_UNUSED( permissions );
    Q_UNUSED( flags );

    error( ERR_UNSUPPORTED_ACTION, i18n( "Cannot put URL '%1' into a tag without a source URL", url.prettyUrl() ) );
}


void Nepomuk::TagsProtocol::copy( const KUrl& src, const KUrl& dest, int permissions, KIO::JobFlags flags )
{
    kDebug();
    QString tag, path;
    if ( splitUrl( src, tag, path ) ) {
        QString newTag, newPath;
        if ( splitUrl( dest, newTag, newPath ) ) {
            if ( newTag.isEmpty() ) {
                ForwardingSlaveBase::copy( src, dest, permissions, flags );
            }
            else {
                Resource res( path );
                res.addTag( Tag( newTag ) );
                finished();
            }
        }
    }
}


void Nepomuk::TagsProtocol::rename( const KUrl& src, const KUrl& dest, KIO::JobFlags flags )
{
    kDebug();
    QString tag, path;
    if ( splitUrl( src, tag, path ) ) {
        if ( path.isEmpty() ) {
            QString newTag, newPath;
            if ( splitUrl( dest, newTag, newPath ) ) {
                if ( !newTag.isEmpty() ) {
                    // rename tag
                    Tag theTag( tag );
                    theTag.setLabel( newTag );
                    theTag.setIdentifiers( QStringList( newTag ) );
                    finished();
                }
                else {
                    error( ERR_CANNOT_RENAME, i18n( "No new tag name specified in URL %1." ).arg( dest.url() ) );
                }
            }
        }
        else {
            ForwardingSlaveBase::rename( src, dest, flags );
        }
    }
}


void Nepomuk::TagsProtocol::del( const KUrl& url, bool isfile )
{
    kDebug();
    QString tag, path;
    if ( splitUrl( url, tag, path ) ) {
        if ( tag.isEmpty() ) {
            ForwardingSlaveBase::del( url, isfile );
        }
        else {
            if ( path.isEmpty() ) {
                // delete the tag altogether
                Tag( tag ).remove();
                finished();
            }
            else {
                // "untag" the file
                Resource res( path );
                QList<Tag> tags = res.tags();
                tags.removeAll( Tag( tag ) );
                res.setTags( tags );
                finished();
            }
        }
    }
}


void Nepomuk::TagsProtocol::mimetype( const KUrl& url )
{
    kDebug();
    QString tag;
    QString path = url.path();
    if ( path.isEmpty() || path == "/" ) {
        mimeType( "inode/directory" );
        finished();
    }
    else if ( splitUrl( url, tag, path ) ) {
        if ( path.isEmpty() ) {
            mimeType( "inode/directory" );
            finished();
        }
        else {
            ForwardingSlaveBase::mimetype( url );
        }
    }
}


void Nepomuk::TagsProtocol::stat( const KUrl& url )
{
    kDebug();
    QString path = url.path();
    if ( path.isEmpty() || path == "/" ) {
        //
        // stat the root path
        //
        KIO::UDSEntry uds;
        uds.insert( KIO::UDSEntry::UDS_NAME, QString::fromLatin1( "/" ) );
        uds.insert( KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR );
        uds.insert( KIO::UDSEntry::UDS_MIME_TYPE, QString::fromLatin1( "inode/directory" ) );

        statEntry( uds );
        finished();
    }
    else {
        QString tag;
        if ( splitUrl( url, tag, path ) ) {
            if ( path.isEmpty() ) {
                statEntry( createUDSEntryForTag( Tag( tag ) ) );
            }
            else {
                KUrl newUrl;
                rewriteUrl( url, newUrl );
                statEntry( statFile( newUrl ) );
            }
        }
        finished();
    }
}


bool Nepomuk::TagsProtocol::rewriteUrl( const KUrl& url, KUrl& newURL )
{
    kDebug();
    QString tag, path;
    if ( splitUrl( url, tag, path, false ) ) {
        if ( !tag.isEmpty() ) {
            // HACK
            // FIXME: This is no solution, especially because path is not unique!
            Tag t( tag );
            foreach( Resource res, t.tagOf() ) {
                if ( res.hasProperty( Soprano::Vocabulary::Xesam::url().toString() ) ) {
                    QString p = res.property( Soprano::Vocabulary::Xesam::url().toString() ).toStringList().first();
                    if ( p.endsWith( path ) ) {
                        newURL = p;
                        return true;
                    }
                }
            }
        }

        newURL = path;
        return true;
    }
    else {
        return false;
    }
}


bool Nepomuk::TagsProtocol::splitUrl( const KUrl& url, QString& tag, QString& path, bool signalError )
{
    kDebug();
    path = url.path().mid( 1 );
    int pos = path.indexOf( '/' );
    if ( pos > 0 ) {
        tag = path.left( pos );
        path = path.mid( pos );
        if ( path.endsWith( "/" ) )
            path.truncate( path.length()-1 );
        return true;
    }
    else if ( !path.isEmpty() ) {
        tag = path;
        path = QString();
        return true;
    }
    else {
        if ( signalError ) {
            error( ERR_MALFORMED_URL, i18n( "Unable to handle URL %1" ).arg( url.path() ) );
        }

        return false;
    }
}



void Nepomuk::TagsProtocol::listTags()
{
    kDebug();
    QList<Tag> tags = Tag::allTags();

//    totalSize( tags.count() );

    for ( QList<Tag>::iterator it = tags.begin();
          it != tags.end(); ++it ) {
        listEntry( createUDSEntryForTag( *it ), false );
    }

    listEntry( KIO::UDSEntry(), true );

    finished();
}


void Nepomuk::TagsProtocol::listTag( const QString& tag )
{
    kDebug() << ": " << tag;
    m_currentTag = tag;

    Tag theTag( tag );
    QList<Resource> taggedOnes = theTag.tagOf();

    // FIXME: use the search io slave code for resource listing
    // FIXME: why can't we just use ForwardingSlaveBase to list result files? Probably the same reason as in the search io slave?
    for ( QList<Resource>::const_iterator it = taggedOnes.constBegin();
          it != taggedOnes.constEnd(); ++it ) {
        const Resource& res = *it;
        if ( res.hasProperty( Soprano::Vocabulary::Xesam::url().toString() ) ) {
            listEntry( statFile( KUrl( res.property( Soprano::Vocabulary::Xesam::url().toString() ).toStringList().first() ) ), false );
        }
    }

    listEntry( KIO::UDSEntry(), true );
    finished();
}


KIO::UDSEntry Nepomuk::TagsProtocol::statFile( const KUrl& url )
{
    m_currentUrl = url;
    m_currentUDS.clear();

    KIO::StatJob* job = KIO::stat( m_currentUrl, KIO::HideProgressInfo );
    connect( job, SIGNAL( result(KJob *) ),
             this, SLOT( slotResult(KJob *) ) );
    connect( job, SIGNAL( redirection(KIO::Job *, const KUrl &) ),
             this, SLOT( slotRedirection(KIO::Job *, const KUrl &) ) );

    m_eventLoop.exec();

    return m_currentUDS;
}


// taken from ForwardingSlaveBase
void Nepomuk::TagsProtocol::slotResult( KJob* job )
{
    kDebug();
    if ( job->error() != 0 ) {
        error( job->error(), job->errorText() );
    }
    else {
        KIO::StatJob* stat_job = qobject_cast<KIO::StatJob*>(job);
        if ( stat_job != 0 ) {
            // FIXME: The name is not unique here! This is a problem especially for
            // rewriteUrl since then the URL is also not unique
            m_currentUDS = stat_job->statResult();
            m_currentUDS.insert( KIO::UDSEntry::UDS_LOCAL_PATH, m_currentUrl.path() );
            m_currentUDS.insert( KIO::UDSEntry::UDS_TARGET_URL, m_currentUrl.url() );
        }
    }

    m_eventLoop.exit();
}


// taken from ForwardingSlaveBase
void Nepomuk::TagsProtocol::slotRedirection( KIO::Job *job, const KUrl& url )
{
    kDebug();
    redirection(url);

    // We've been redirected stop everything.
    job->kill( KJob::Quietly );
    finished();

    m_eventLoop.exit();
}



extern "C"
{
    KDE_EXPORT int kdemain( int argc, char **argv )
    {
        KAboutData about( "kio_tags", 0, ki18n("kio_tags"), 0);
        KCmdLineArgs::init( &about );

        KApplication app;

        kDebug(7102) << "Starting tags slave " << getpid();

        if (argc != 4) {
            kError() << "Usage: kio_tags protocol domain-socket1 domain-socket2";
            exit(-1);
        }

        if ( Nepomuk::ResourceManager::instance()->init() ) {
            kError() << "Unable to initialized KMetaData.";
            exit( -1 );
        }

        Nepomuk::TagsProtocol slave(argv[2], argv[3]);
        slave.dispatchLoop();

        kDebug(7102) << "Tags slave Done";

        return 0;
    }
}

#include "kio_tags.moc"
