/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2008 Adrien Bustany <madcat@mymadcat.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "nepomukmenuplugin.h"
#include <konq_popupmenuinformation.h>
#include <klocalizedstring.h>
#include <KActionMenu>

// Used to retrieve the title of the web page
#include <KHTMLView>
#include <KHTMLPart>
#include <dom/html_document.h>
#include <dom/dom_string.h>
#include <KTemporaryFile>
#include <KDebug>

#include <QtGui/QWidgetAction>
#include <QtDBus/QDBusMessage>
#include <QtDBus/QDBusConnection>

#include <nepomuk/resourcemanager.h>
#include <nepomuk/kratingwidget.h>
#include <nepomuk/tag.h>
#include <Nepomuk/Variant>


#include <soprano/soprano.h>
#include <soprano/asyncmodel.h>

#include "newtagdialog.h"

#define PIMO_WEBPAGE_TYPE "http://www.semanticdesktop.org/ontologies/2007/11/01/pimo#Document"
#define PIMO_GROUNDINGOCCURRENCE "http://www.semanticdesktop.org/ontologies/2007/11/01/pimo#groundingOccurrence"
#define NFO_WEBSITE_TYPE "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#Website"

NepomukMenuPlugin::NepomukMenuPlugin(QObject* parent, const QVariantList&) : KonqPopupMenuPlugin(parent)
{
	m_rateAction = new QWidgetAction(this);
	m_ratingWidget = new KRatingWidget(); // WidgetAction will take ownership of this widget
	connect(m_ratingWidget, SIGNAL(ratingChanged(unsigned int)), this, SLOT(ratingChanged(unsigned int)));
	m_rateAction->setDefaultWidget(m_ratingWidget);

	m_addTagAction = new KAction(i18n("Add a new tag..."), this);
	connect(m_addTagAction, SIGNAL(triggered(bool)), this, SLOT(addNewTag()));

	// Set up the soprano link
	m_soprano = new Soprano::Util::AsyncModel(Nepomuk::ResourceManager::instance()->mainModel());
	m_soprano->setMode(Soprano::Util::AsyncModel::MultiThreaded);
}

NepomukMenuPlugin::~NepomukMenuPlugin()
{
	delete m_soprano;
}

void NepomukMenuPlugin::setup(KActionCollection* collection, const KonqPopupMenuInformation& info, QMenu* menu)
{
	if(info.items().size() != 1) return; // We only want 1 item selections

	m_currentUri = info.items().urlList().first();

	QString query = QString("select ?pimoid where"
							"{ <%1> a <" NFO_WEBSITE_TYPE "> ."
							"?pimoid <" PIMO_GROUNDINGOCCURRENCE "> <%1> ."
							"?pimoid a <" PIMO_WEBPAGE_TYPE "> . }")
					.arg(QString::fromAscii( m_currentUri.toEncoded()) );
	kDebug() << query;
	Soprano::Util::AsyncResult* result = m_soprano->executeQueryAsync(query, Soprano::Query::QueryLanguageSparql);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(fillMenu(Soprano::Util::AsyncResult*)));

	// Rating part
	KActionMenu* rateMenu = new KActionMenu(i18n("Rate this page"), collection);

	rateMenu->addAction(m_rateAction);
	menu->addAction(rateMenu);

	// Tagging part
	m_actionUrl.clear();
	m_actionChecked.clear();

	KActionMenu* tagMenu = new KActionMenu(i18n("Tag this page"), collection);
	QList<Nepomuk::Tag> allTags = Nepomuk::Tag::allTags();
	foreach(const Nepomuk::Tag& tag, allTags) {
		KAction* action = new KAction(tag.label(), tagMenu); // Will be destroyed when the menu gets closed
		action->setCheckable(true);
		tagMenu->addAction(action);
		m_actionUrl.insert(action, tag.resourceUri());
		connect(action, SIGNAL(toggled(bool)), this, SLOT(tagCallback()));
	}
	tagMenu->addAction(m_addTagAction);
	menu->addAction(tagMenu);

	m_view = qobject_cast<KHTMLView*>(info.parentWidget());
}

Nepomuk::Resource NepomukMenuPlugin::saveCurrentPage()
{
	Nepomuk::Resource nfoResource(m_currentUri, QUrl::fromEncoded(NFO_WEBSITE_TYPE, QUrl::StrictMode));

	// pimoResource is invalid by default, which means that Nepomuk will create a new resource on setProperty
	// it can also be set to an already existing resource by fillMenu
	if(!m_pimoResource.isValid()) {
		m_pimoResource.setTypes( QList<QUrl>() << QUrl::fromEncoded(PIMO_WEBPAGE_TYPE, QUrl::StrictMode));
		m_pimoResource.setProperty(QUrl(PIMO_GROUNDINGOCCURRENCE), Nepomuk::Variant(nfoResource));
	}

	// Try to get the page title by accessing the KHtml component. If we fail, use the URL as a label.
	if(m_view) { // if Konqueror is in web browser
		m_pimoResource.setLabel(m_view->part()->htmlDocument().title().string());

		KTemporaryFile tmpFile;
		tmpFile.setAutoRemove( false );
		tmpFile.open();
		QTextStream s( &tmpFile );
		s << m_view->part()->htmlDocument().toHTML();
		QDBusMessage msg = QDBusMessage::createMethodCall( "org.kde.nepomuk.services.nepomukstrigiservice",
														   "/nepomukstrigiservice",
														   "org.kde.nepomuk.Strigi",
														   "analyzeResourceFromTempFileAndDeleteTempFile" );
		msg.setArguments( QList<QVariant>()
						  << QString::fromAscii( nfoResource.resourceUri().toEncoded() )
						  << QDateTime::currentDateTime().toTime_t()
						  << tmpFile.fileName() );
		QDBusConnection::sessionBus().send( msg );
	} else { // Konqueror is in file manager mode / other
		m_pimoResource.setLabel(m_currentUri.toString());
	}

	return m_pimoResource;
}

void NepomukMenuPlugin::fillMenu(Soprano::Util::AsyncResult* result)
{
	kDebug();
	Soprano::QueryResultIterator it = result->value().value<Soprano::QueryResultIterator>();
 
	if( it.next() ) {
		m_pimoResource = Nepomuk::Resource( it.binding( "pimoid" ).uri() );
		it.close();
 
		// Load rating
		m_ratingWidget->setRating(m_pimoResource.rating());
 
		// Load tags
		foreach(const Nepomuk::Tag& tag, m_pimoResource.tags()) {
			KAction* action = m_actionUrl.key(tag.resourceUri());
			action->setChecked(true);
			m_actionChecked[action] = true;
		}
	}
	else {
		kDebug() << "Resource does not exist yet";
	}
}

void NepomukMenuPlugin::ratingChanged(unsigned int rating)
{
	Nepomuk::Resource res = saveCurrentPage();
	res.setRating(rating);
}

void NepomukMenuPlugin::tagCallback()
{
	bool hasChanged = false;
	Nepomuk::Resource res = saveCurrentPage();
	QList<Nepomuk::Tag> tags = res.tags();

	foreach(KAction* action, m_actionUrl.keys()) {
		if(action->isChecked() == m_actionChecked[action]) continue;
		hasChanged = true;
		Nepomuk::Tag tag(m_actionUrl[action]);
		if(action->isChecked()) {
			tags.append(tag);
		} else {
			tags.removeOne(tag);
		}
	}

	if(hasChanged) {
		res.setTags(tags);
	}
}

void NepomukMenuPlugin::addNewTag()
{
	NewTagDialog::createTag();
}

Q_DECLARE_METATYPE(Soprano::QueryResultIterator);
#include "nepomukmenuplugin.moc"
