/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2008 Adrien Bustany <madcat@mymadcat.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include <KJob>
#include <kio/jobclasses.h>
#include <KLocalizedString>

#include "linkwidget.h"
#include "resourcemodel.h"
#include "resourcedelegate.h"
#include "relationmodel.h"
#include "annotationpluginfactory.h"
#include "annotationresource.h"

using namespace Nepomuk;

LinkWidget::LinkWidget(QWidget* parent) : KLineEdit(parent)
{
	setClearButtonShown(true);
	m_resourceView = new QListView();
	m_resourceView->setItemDelegate(new ResourceDelegate(this));
	m_resourceView->setWindowFlags(Qt::X11BypassWindowManagerHint);
	connect(m_resourceView, SIGNAL(activated(const QModelIndex&)), this, SLOT(itemActivated(const QModelIndex&)));

	m_resources = new ResourceModel(this);
	m_resourceView->setModel(m_resources);
	m_resourceView->resize(300, 400);

	m_relations = new RelationModel(this);
	m_sortedRelations = new QSortFilterProxyModel(this);
	m_sortedRelations->setSortRole(RelationModel::Relevance);
	m_sortedRelations->setDynamicSortFilter(true);
	m_sortedRelations->sort(0, Qt::DescendingOrder);
	m_sortedRelations->setSourceModel(m_relations);

	m_escapeShortcut = new QShortcut(QKeySequence(Qt::Key_Escape), this);
	connect(m_escapeShortcut, SIGNAL(activated()), this, SLOT(resetWidget()));

	connect(this, SIGNAL(textChanged(const QString&)), this, SLOT(textChanged(const QString&)));

	setupAnnotationPlugins();

	setClickMessage(i18n("Link to..."));
	m_position = West;
	placeLists();
}

LinkWidget::~LinkWidget()
{
	if(m_resourceView) m_resourceView->deleteLater();
}

void LinkWidget::setPopupParent(QWidget* parent)
{
	m_resourceView->setParent(parent);
}

void LinkWidget::setPopupPosition(Position pos) {
	m_position = pos;
}

QWidget* LinkWidget::popup()
{
	return m_resourceView;
}

void LinkWidget::setFile(const KUrl& url)
{
	resetWidget();
	m_currentFile = url;
	m_relations->setMimeType(QString());
	KIO::MimetypeJob* job = KIO::mimetype(url);
	connect(job, SIGNAL(mimetype(KIO::Job*, const QString&)), this, SLOT(mimetype_cb(KIO::Job*, const QString&)));
}

void LinkWidget::setupAnnotationPlugins()
{
	AnnotationPluginFactory* factory = AnnotationPluginFactory::instance();

	foreach(AnnotationPlugin* p, factory->getAllPlugins()) {
		m_dataProviders.append(p);
		connect(p, SIGNAL(finished()), this, SIGNAL(finished()));
	}

	m_resources->setAnnotationPlugins(m_dataProviders);
}

void LinkWidget::placeLists()
{
	// Don't place the list if it has been parented
	if(m_resourceView->parent()) return;

	QPoint coords = parentWidget() ? parentWidget()->mapToGlobal(pos()) : mapToGlobal(pos());
	switch (m_position) {
		case North:
			m_resourceView->move(coords - QPoint(0, m_resourceView->geometry().height()));
			break;
		case East:
			m_resourceView->move(coords + QPoint(geometry().width(), 0));
			break;
		case South:
			m_resourceView->move(coords + QPoint(0, geometry().height()));
			break;
		case West:
			m_resourceView->move(coords - QPoint(m_resourceView->geometry().width(), 0));
			break;
	}
}

void LinkWidget::textChanged(const QString& text)
{
	if(text.isEmpty()) {
		resetWidget();
		return;
	}
	m_resourceView->show();
	placeLists();
	setFocus(Qt::OtherFocusReason);
	m_resources->setFilter(text);
}

void LinkWidget::itemActivated(const QModelIndex& index)
{
	// If the user selected a resource
	if(m_resourceView->model() == m_resources) {
		m_currentResourceUri = index.data(ResourceModel::Uri).toString();
		m_relations->clear();
		// And additionnaly, the relations provided by the AnnotationPlugin which created the object
		AnnotationPlugin* creator = m_resources->plugin(index);
		m_relations->fetchProperties(creator);
		m_resourceView->setModel(m_sortedRelations);
		return;
	}

	// If the user selected a relation
	if(m_resourceView->model() == m_sortedRelations) {
		QModelIndex sourceIndex = m_sortedRelations->mapToSource(index);
		m_currentRelationUri = sourceIndex.data(ResourceModel::Uri).toUrl();
		AnnotationPlugin* creator = m_relations->plugin(sourceIndex);
		emit busy();
		creator->addRelation(m_currentRelationUri, m_currentFile, m_currentResourceUri);
		resetWidget();
		return;
	}
}

void LinkWidget::mimetype_cb(KIO::Job* /*job*/, const QString& type)
{
	m_relations->setMimeType(type);
}

void LinkWidget::resetWidget()
{
	m_resourceView->hide();
	m_resourceView->setModel(m_resources);
	clear();
}

void LinkWidget::closeEvent(QCloseEvent* event)
{
	m_resourceView->close();
	event->accept();
	close();
}

void LinkWidget::keyPressEvent(QKeyEvent* event)
{
	switch(event->key()) {
		case Qt::Key_Up:
		case Qt::Key_Down:
			QCoreApplication::sendEvent(m_resourceView, event);
			break;
		default:
			KLineEdit::keyPressEvent(event);
	}
}

void LinkWidget::keyReleaseEvent(QKeyEvent* event)
{
	switch(event->key()) {
		case Qt::Key_Up:
		case Qt::Key_Down:
			QCoreApplication::sendEvent(m_resourceView, event);
			break;
		case Qt::Key_Enter:
		case Qt::Key_Return:
			itemActivated(m_resourceView->currentIndex());
			break;
		default:
			KLineEdit::keyReleaseEvent(event);
	}
}

#include "linkwidget.moc"
