/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2008 Adrien Bustany <madcat@mymadcat.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "resourcemodel.h"
#include "annotationplugin.h"
#include "annotationresource.h"
#include "annotationresult.h"
#include <Nepomuk/Resource>
#include <kiconloader.h>

using namespace Nepomuk;

ResourceModel::ResourceModel(QObject* parent) : QAbstractListModel(parent)
{
}

int ResourceModel::rowCount(const QModelIndex& /*parent*/) const
{
	return m_resources.size();
}

QVariant ResourceModel::data(const QModelIndex& index, int role) const
{
	if(index.row() >= m_resources.size()) return QVariant();
	switch(role) {
		case Qt::DisplayRole:
			return m_resources.at(index.row()).label();

		case Qt::DecorationRole:
			return m_icons.value(m_resources.at(index.row()).resource().uri());

		case ResourceModel::Description:
			return m_resources.at(index.row()).description();

		case ResourceModel::Uri:
			return m_resources.at(index.row()).resource().uri();

		default:
			return QVariant();
	}
}

AnnotationPlugin* ResourceModel::plugin(const QModelIndex& index) const
{
	return m_resources.at(index.row()).parent();
}

void ResourceModel::setFilter(const QString& filter)
{
	beginRemoveRows(QModelIndex(), 0, m_resources.size()-1);
	m_resources.clear();
	endRemoveRows();

	foreach(AnnotationPlugin* p, m_annotationPlugins) {
		Nepomuk::AnnotationResult* r = p->getResources(filter);
		connect(r, SIGNAL(ready()), this, SLOT(result_cb()));
		r->emitStatus();
	}
}

void ResourceModel::setAnnotationPlugins(const QList<AnnotationPlugin*>& plugins)
{
	if(!m_annotationPlugins.isEmpty()) {
		foreach(AnnotationPlugin* p, m_annotationPlugins) {
			p->disconnect(this);
		}
	}

	m_annotationPlugins = plugins;
}

void ResourceModel::result_cb()
{
	Nepomuk::AnnotationResult* r = qobject_cast<Nepomuk::AnnotationResult*>(sender());
	if(!r)
		return;

	if(r->resources().size() == 0) return;
	// Fill the icon cache
	foreach(const Nepomuk::AnnotationResource& res, r->resources()) {
		if(res.icon().type() == QVariant::String) {
			m_icons.insert(res.resource().uri(), DesktopIcon(res.icon().toString(), 64));
		} else if (res.icon().type() == QVariant::Image) {
			m_icons.insert(res.resource().uri(), QPixmap::fromImage(res.icon().value<QImage>()));
		}
	}
	beginInsertRows(QModelIndex(), m_resources.size(), m_resources.size()+r->resources().size()-1);
	m_resources += r->resources();
	endInsertRows();

	delete r;
}

#include "resourcemodel.moc"
