/*
   Copyright (C) 2008 by Sebastian Trueg <trueg at kde.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "mainwindow.h"
#include "nsqclientmodel.h"
#include "query.h"
#include "searchcore.h"
#include "queryparser.h"
#include "searchhitmodel.h"
#include "searchhitview.h"

#include <QtGui/QLabel>
#include <QtGui/QListView>
#include <QtGui/QHBoxLayout>
#include <QtGui/QVBoxLayout>
#include <QtGui/QInputDialog>
#include <QtGui/QSplitter>
#include <QtCore/QDateTime>

#include <KIcon>
#include <KLocale>
#include <KLineEdit>
#include <KCompletion>
#include <KMessageBox>
#include <KDebug>
#include <KMenu>

#include <DNSSD/RemoteService>

#include <Soprano/Model>
#include <Soprano/LiteralValue>
#include <Soprano/StatementIterator>
#include <Soprano/Vocabulary/RDF>
#include <Soprano/Vocabulary/NRL>
#include <Soprano/Vocabulary/NAO>

#include <nepomuk/resourcemanager.h>


MainWindow::MainWindow()
    : QWidget( 0 ),
      m_lastRemoteQueryModel( 0 )
{
    setWindowTitle( i18n( "Nepomuk Social Query Client" ) );

    QSplitter* splitter = new QSplitter( this );
    splitter->setOrientation( Qt::Vertical );

    m_buddyView = new QListView( splitter );
    splitter->addWidget( m_buddyView );
    m_buddyView->setViewMode( QListView::IconMode );
    m_buddyView->setIconSize( QSize( 48, 48 ) );

    m_buddyModel = new Nepomuk::SocialQueryClientModel( m_buddyView );
    m_buddyView->setModel( m_buddyModel );
    m_buddyView->setSpacing( 10 );

    QVBoxLayout* mainLay = new QVBoxLayout( this );
    mainLay->setMargin( 0 );
    mainLay->setSpacing( 0 );
    QHBoxLayout* titleLay = new QHBoxLayout;
    titleLay->setMargin( 10 );
    titleLay->setSpacing( 20 );

    QLabel* iconLabel = new QLabel( this );
    iconLabel->setPixmap( KIcon( "nepomuk" ).pixmap( QSize( 64, 64 ) ) );
    QLabel* titleLabel = new QLabel( "<qt>Nepomuk <h2 style=\"margin-top:2px;\">Social Query Client</h2>", this );
    titleLabel->setAlignment( Qt::AlignTop );

    m_resultsView = new Nepomuk::SearchHitView( splitter );
    m_resultsView->setContextMenuPolicy( Qt::CustomContextMenu );
    splitter->addWidget( m_resultsView );

    titleLay->addWidget( iconLabel );
    titleLay->addWidget( titleLabel, 1 );
    mainLay->addLayout( titleLay );
    mainLay->addWidget( splitter );

    m_searchCore = new Nepomuk::Search::SearchCore( this );

    connect( m_searchCore, SIGNAL( newResult( const QUrl&, double ) ),
             m_resultsView, SLOT( updateResource( const QUrl&, double ) ) );
    connect( m_searchCore, SIGNAL( scoreChanged( const QUrl&, double ) ),
             m_resultsView, SLOT( updateResource( const QUrl&, double ) ) );

    connect( m_buddyView, SIGNAL( doubleClicked(const QModelIndex& ) ),
             this, SLOT( slotItemActivated( const QModelIndex& ) ) );
    connect( m_resultsView, SIGNAL( customContextMenuRequested( const QPoint& ) ),
             this, SLOT( slotContextMenu( const QPoint& ) ) );
}


MainWindow::~MainWindow()
{
    Nepomuk::ResourceManager::instance()->setOverrideMainModel( 0 );
    delete m_lastRemoteQueryModel;
}


Q_DECLARE_METATYPE( Soprano::Model* )

void MainWindow::slotItemActivated( const QModelIndex& index )
{
    Nepomuk::ResourceManager::instance()->setOverrideMainModel( 0 );
    delete m_lastRemoteQueryModel;
    m_lastRemoteQueryModel = 0;

    QString query = QInputDialog::getText( this,
                                           i18n( "Query %1", index.data( Nepomuk::SocialQueryClientModel::NSQNameRole ).toString() ),
                                           i18n( "Please enter the query:" ) );
    if ( !query.isEmpty() ) {
        if ( Soprano::Model* model = index.data( Nepomuk::SocialQueryClientModel::NSQModelRole ).value<Soprano::Model*>() ) {
            m_lastRemoteQueryModel = model;
            Nepomuk::ResourceManager::instance()->setOverrideMainModel( model );

            kDebug() << "Searching for" << query;

            Nepomuk::Search::Query q = Nepomuk::Search::QueryParser::parseQuery( query );
            q.setLimit( 50 );

            kDebug() << "Search:" << q;

            m_searchCore->query( q );
        }
        else {
            KMessageBox::error( this, i18n( "Failed to contact remote NSQ service." ) );
        }
    }
}


void MainWindow::slotContextMenu( const QPoint& pos )
{
    QModelIndex index = m_resultsView->indexAt( pos );
    if ( index.isValid() ) {
        QAction* actionImport = new QAction( KIcon( "document-import" ), i18n( "Import Resource" ), this );
        if ( QMenu::exec( QList<QAction*>() << actionImport, m_resultsView->viewport()->mapToGlobal( pos ) ) == actionImport ) {
            importResource( index.data( Nepomuk::SearchHitModel::ResourceRole ).toUrl() );
        }
        delete actionImport;
    }
}


void MainWindow::importResource( const QUrl& res )
{
    if ( m_lastRemoteQueryModel ) {
        kDebug() << "Importing resource" << res;
        Nepomuk::ResourceManager::instance()->setOverrideMainModel( 0 );
        int cnt = 0;
        QUrl graph = Nepomuk::ResourceManager::instance()->generateUniqueUri();
        Soprano::StatementIterator it = m_lastRemoteQueryModel->listStatements( res, Soprano::Node(), Soprano::Node() );
        // FIXME: Use the filter model which creates NRL graphs automatically here
        while ( it.next() ) {
            ++cnt;
            Soprano::Statement s = *it;
            s.setContext( graph );
            Nepomuk::ResourceManager::instance()->mainModel()->addStatement( s );
        }
        if ( cnt ) {
            Nepomuk::ResourceManager::instance()->mainModel()->addStatement( graph,
                                                                             Soprano::Vocabulary::RDF::type(),
                                                                             Soprano::Vocabulary::NRL::InstanceBase() );
            Nepomuk::ResourceManager::instance()->mainModel()->addStatement( graph,
                                                                             Soprano::Vocabulary::NAO::created(),
                                                                             Soprano::LiteralValue( QDateTime::currentDateTime() ) );
        }
        Nepomuk::ResourceManager::instance()->setOverrideMainModel( m_lastRemoteQueryModel );
    }
}

#include "mainwindow.moc"
