/*
   Copyright (C) 2008 by Sebastian Trueg <trueg at kde.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "nsqclientmodel.h"

#include <DNSSD/ServiceBrowser>
#include <DNSSD/RemoteService>

#include <QtNetwork/QHostInfo>

#include <Soprano/Model>
#include <Soprano/StatementIterator>
#include <Soprano/Client/TcpClient>
#include <Soprano/Vocabulary/RDFS>

#include <KIcon>
#include <KDebug>

Q_DECLARE_METATYPE( Soprano::Model* )


class Nepomuk::SocialQueryClientModel::Private
{
public:
    Private( SocialQueryClientModel* parent )
        : q( parent ) {
    }

    DNSSD::ServiceBrowser* dnssdBrowser;
    QList<QPair<QString, DNSSD::RemoteService::Ptr> > nsqServices;

    void rebuildServiceList();
    void _k_serviceAdded( DNSSD::RemoteService::Ptr );
    void _k_serviceRemoved( DNSSD::RemoteService::Ptr );

    Soprano::Model* createModel( DNSSD::RemoteService::Ptr service, const QString& name );

private:
    SocialQueryClientModel* q;
};


Soprano::Model* Nepomuk::SocialQueryClientModel::Private::createModel( DNSSD::RemoteService::Ptr service, const QString& name )
{
    Soprano::Client::TcpClient* sopranoTcpClient = new Soprano::Client::TcpClient();
    QHostInfo hostInfo = QHostInfo::fromName( service->hostName() );
    if ( hostInfo.addresses().count() ) {
        if ( sopranoTcpClient->connect( hostInfo.addresses().first(), service->port() ) ) {
            if ( Soprano::Model* model = sopranoTcpClient->createModel( name ) ) {
                // A little trick: make sure the client it deleted if the mdoel is
                sopranoTcpClient->setParent( model );
                return model;
            }
        }
        else {
            kDebug() << "Failed to connect to service" << service->serviceName() << "at"
                     << QString( "%1:%2" ).arg( hostInfo.addresses().first().toString() ).arg( service->port() );
        }
    }
    else {
        kDebug() << "Unable to resolve hostname" << service->hostName();
    }

    delete sopranoTcpClient;
    return 0;
}


void Nepomuk::SocialQueryClientModel::Private::rebuildServiceList()
{
    nsqServices.clear();
    foreach( DNSSD::RemoteService::Ptr service, dnssdBrowser->services() ) {
        // retrieve name
        if ( Soprano::Model* model = createModel( service, "nsq-personal-data" ) ) {
            Soprano::StatementIterator it = model->listStatements( QUrl( "http://nepomuk.kde.org/nsqd/onto#Instance" ),
                                                                   Soprano::Vocabulary::RDFS::label(),
                                                                   Soprano::Node() );
            if ( it.next() ) {
                kDebug() << "Successfully contacted service" << service->serviceName() << "by" << it.current().object().toString();
                nsqServices << qMakePair<QString, DNSSD::RemoteService::Ptr>( it.current().object().toString(), service );
            }
            delete model;
        }
    }

    q->reset();
}


void Nepomuk::SocialQueryClientModel::Private::_k_serviceAdded( DNSSD::RemoteService::Ptr )
{
    // the easy way
    rebuildServiceList();
}


void Nepomuk::SocialQueryClientModel::Private::_k_serviceRemoved( DNSSD::RemoteService::Ptr )
{
    // the easy way
    rebuildServiceList();
}


Nepomuk::SocialQueryClientModel::SocialQueryClientModel( QObject* parent )
    : QAbstractItemModel( parent ),
      d( new Private( this ) )
{
    d->dnssdBrowser = new DNSSD::ServiceBrowser( "_nepomuk._tcp", true /* resolve all services */ );
    connect( d->dnssdBrowser, SIGNAL(serviceAdded(DNSSD::RemoteService::Ptr)),
             this, SLOT(_k_serviceAdded(DNSSD::RemoteService::Ptr) ) );
    connect( d->dnssdBrowser, SIGNAL(serviceRemoved(DNSSD::RemoteService::Ptr)),
             this, SLOT(_k_serviceRemoved(DNSSD::RemoteService::Ptr) ) );
    d->dnssdBrowser->startBrowse();
}


Nepomuk::SocialQueryClientModel::~SocialQueryClientModel()
{
    delete d->dnssdBrowser;
    delete d;
}


int Nepomuk::SocialQueryClientModel::columnCount( const QModelIndex& ) const
{
    return 1;
}


QVariant Nepomuk::SocialQueryClientModel::data( const QModelIndex& index, int role ) const
{
    if ( index.column() == 0 &&
         index.row() >= 0 &&
         index.row() < d->nsqServices.count() ) {
        switch( role ) {
        case Qt::DecorationRole:
            return KIcon( "personal" );

        case Qt::DisplayRole:
        case NSQNameRole:
            return d->nsqServices[index.row()].first;

        case NSQServiceRole:
            return qVariantFromValue( d->nsqServices[index.row()].second );

        case NSQModelRole:
            return qVariantFromValue( d->createModel( d->nsqServices[index.row()].second, "main" ) );

        default:
            return QVariant();
        }
    }
    else {
        return QVariant();
    }
}


QModelIndex Nepomuk::SocialQueryClientModel::index( int row, int column, const QModelIndex& parent ) const
{
    Q_UNUSED( parent );
    return createIndex( row, column, 0 );
}


QModelIndex Nepomuk::SocialQueryClientModel::parent( const QModelIndex& index ) const
{
    Q_UNUSED( index );
    return QModelIndex();
}


int Nepomuk::SocialQueryClientModel::rowCount( const QModelIndex& parent ) const
{
    return d->nsqServices.count();
}

#include "nsqclientmodel.moc"
