/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2008 Adrien Bustany <madcat@mymadcat.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "nepomukproxy.h"
#include <nepomuk/resource.h>
#include <nepomuk/resourcemanager.h> // Needed for uri generation
#include <soprano/soprano.h>
#include <soprano/asyncmodel.h>

#define NCO "http://www.semanticdesktop.org/ontologies/2007/03/22/nco#"
#define NIR "http://www.semanticdesktop.org/ontologies/2008/07/04/nir#"

using namespace Nepomuk;

// Singleton initialization
NepomukProxy *NepomukProxy::m_instance = 0;

// Private d pointer
class NepomukProxy::Private {
public:
	unsigned int addContactCompletion;
	unsigned int addGroupCompletion;
	unsigned int addRegionCompletion;
	unsigned int updateRegionCompletion;
	unsigned int removeRegionCompletion;
	Soprano::Util::AsyncModel* soprano;
	// Used to store data for consecutive queries (ie one query needs data from the previous one)
	QHash<Soprano::Util::AsyncResult*, QVariantMap> queryData;

};

NepomukProxy::~NepomukProxy()
{
	delete d;
}

NepomukProxy* NepomukProxy::getInstance()
{
	if(!m_instance) {
		m_instance = new NepomukProxy();
		m_instance->setParent(qApp);
	}

	return m_instance;
}

QString NepomukProxy::addContact(const QString& fullname, const QString& group)
{
	// We've at least two things to do : create the resource and add a fullname
	d->addContactCompletion +=2;

	Soprano::Node personId;
	Soprano::Node predicate;
	Soprano::Node object;
	Soprano::Util::AsyncResult* result;

	// Create a new resource of type nco:PersonContact
	personId = QUrl(Nepomuk::ResourceManager::instance()->generateUniqueUri());
	predicate = Soprano::Vocabulary::RDF::type();
	object = QUrl::fromEncoded(NCO "PersonContact", QUrl::StrictMode);
	result = d->soprano->addStatementAsync(personId, predicate, object);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(addContact_cb(Soprano::Util::AsyncResult*)));

	// Add a fullname field to this resource
	predicate = QUrl::fromEncoded(NCO "fullname", QUrl::StrictMode);
	object = Soprano::LiteralValue(fullname);
	result = d->soprano->addStatementAsync(personId, predicate, object);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(addContact_cb(Soprano::Util::AsyncResult*)));

	// If we don't have a group parameter, then we've nothing left to do
	if (group == QString()) {
		return personId.uri().toString();
	}

	d->addContactCompletion ++;

	// Add the person to the group
	predicate = QUrl::fromEncoded(NCO "belongsToGroup", QUrl::StrictMode);
	object = QUrl(group);
	result = d->soprano->addStatementAsync(personId, predicate, object);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(addContact_cb(Soprano::Util::AsyncResult*)));

	return personId.uri().toString();
}

QString NepomukProxy::addGroup(const QString& groupname)
{
	d->addGroupCompletion += 2;

	Soprano::Node groupId;
	Soprano::Node predicate;
	Soprano::Node object;
	Soprano::Util::AsyncResult* result;

	// Create a new resource of type nco:PersonContact
	groupId = QUrl(Nepomuk::ResourceManager::instance()->generateUniqueUri());
	predicate = Soprano::Vocabulary::RDF::type();
	object = QUrl::fromEncoded(NCO "ContactGroup", QUrl::StrictMode);
	result = d->soprano->addStatementAsync(groupId, predicate, object);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(addGroup_cb(Soprano::Util::AsyncResult*)));

	groupId = QUrl(Nepomuk::ResourceManager::instance()->generateUniqueUri());
	predicate = QUrl::fromEncoded(NCO "contactGroupName", QUrl::StrictMode);
	object = Soprano::LiteralValue(groupname);
	result = d->soprano->addStatementAsync(groupId, predicate, object);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(addGroup_cb(Soprano::Util::AsyncResult*)));

	return groupId.uri().toString();
}

void NepomukProxy::addRegion(const QString& path, const QRectF& rect, const QString& uid)
{
	d->addRegionCompletion += 4;

	Soprano::Node subject;
	Soprano::Node predicate;
	Soprano::Node object;
	Soprano::Util::AsyncResult* result;

	// Create the region
	subject = QUrl(Nepomuk::ResourceManager::instance()->generateUniqueUri());
	QUrl regionId = subject.uri(); // Save the uri for later use
	predicate = Soprano::Vocabulary::RDF::type();
	object = QUrl::fromEncoded(NIR "Rectangle", QUrl::StrictMode); // We only create rectangles for now
	result = d->soprano->addStatementAsync(subject, predicate, object);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(addRegion_cb(Soprano::Util::AsyncResult*)));

	// Set the coordinates
	predicate = QUrl::fromEncoded(NIR "coords", QUrl::StrictMode);
	object = Soprano::LiteralValue(QString().sprintf("%f,%f,%f,%f", rect.x(), rect.y(), rect.width(), rect.height()));
	result = d->soprano->addStatementAsync(subject, predicate, object);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(addRegion_cb(Soprano::Util::AsyncResult*)));

	// Set the depicted resource
	predicate = QUrl::fromEncoded(NIR "regionDepicts", QUrl::StrictMode);
	object = QUrl(uid);
	result = d->soprano->addStatementAsync(subject, predicate, object);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(addRegion_cb(Soprano::Util::AsyncResult*)));

	// Link the region to the picture
	subject = QUrl(QString("file://%1").arg(path));
	predicate = QUrl::fromEncoded(NIR "region", QUrl::StrictMode);
	object = regionId;
	result = d->soprano->addStatementAsync(subject, predicate, object);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(addRegion_cb(Soprano::Util::AsyncResult*)));
}

void NepomukProxy::updateRegion(const QString& mid, const QRectF& rect, const QString& uid)
{
	d->updateRegionCompletion += 2;
	// Remove the old coordinates statement
	Soprano::Node subject;
	Soprano::Node predicate;
	Soprano::Node object;
	Soprano::Util::AsyncResult* result;

	QVariantMap userData;
	userData["rect"] = rect;
	userData["uid"] = uid;
	userData["mid"] = mid;

	subject = QUrl(mid);
	predicate = QUrl::fromEncoded(NIR "coords", QUrl::StrictMode);
	object = Soprano::Node(); // Wildcard
	result = d->soprano->removeStatementAsync(subject, predicate, object);
	d->queryData[result] = userData; // We'll need that information later, when we'll insert the updated values
	// We're obliged to do this in two times, launching both operations concurrently could lead to a race condition
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(finishUpdate(Soprano::Util::AsyncResult*)));

	predicate = QUrl::fromEncoded(NIR "regionDepicts", QUrl::StrictMode);
	result = d->soprano->removeStatementAsync(subject, predicate, object);
	d->queryData[result] = userData;
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(finishUpdate(Soprano::Util::AsyncResult*)));
}

void NepomukProxy::removeRegion(const QString& mid)
{
	d->removeRegionCompletion += 4;

	Soprano::Node subject;
	Soprano::Node predicate;
	Soprano::Node object;
	Soprano::Util::AsyncResult* result;

	// Remove the type statement
	subject = QUrl(mid);
	predicate = Soprano::Vocabulary::RDF::type();
	object = Soprano::Node(); // Wildcard (this can be a Rectangle, a Region, an Ellipse...)
	result = d->soprano->removeStatementAsync(subject, predicate, object);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(removeRegion_cb(Soprano::Util::AsyncResult*)));

	// Remove the coords statement
	predicate = QUrl::fromEncoded(NIR "coords", QUrl::StrictMode);
	object = Soprano::Node(); // Wildcard
	result = d->soprano->removeStatementAsync(subject, predicate, object);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(removeRegion_cb(Soprano::Util::AsyncResult*)));

	// Remove the region <-> resource link
	predicate = QUrl::fromEncoded(NIR "regionDepicts", QUrl::StrictMode);
	object = Soprano::Node(); // Wildcard
	result = d->soprano->removeStatementAsync(subject, predicate, object);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(removeRegion_cb(Soprano::Util::AsyncResult*)));

	// Unlink the region from the picture
	subject = Soprano::Node();
	predicate = QUrl::fromEncoded(NIR "region", QUrl::StrictMode);
	object = QUrl(mid);
	result = d->soprano->removeStatementAsync(subject, predicate, object);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(removeRegion_cb(Soprano::Util::AsyncResult*)));

}

void NepomukProxy::getContacts(const QString& group)
{
	QString query = "select ?uid ?gid ?name ?gname where"
					"{ ?uid a <" NCO "PersonContact> ."
					"?uid <" NCO "fullname> ?name .";
	if(group != QString()) {
		query += QString("<%1> a <" NCO "ContactGroup> . ?uid <" NCO "belongsToGroup> <%1> .").arg(group);
	}
	query += "optional { ?uid <" NCO "belongsToGroup> ?gid . "
				"?gid a <" NCO "ContactGroup> ."
				"?gid <" NCO "contactGroupName> ?gname }}";
	qDebug() << query;
	Soprano::Util::AsyncResult* result = d->soprano->executeQueryAsync(query, Soprano::Query::QueryLanguageSparql);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(contacts_cb(Soprano::Util::AsyncResult*)));
}

void NepomukProxy::getGroups()
{
	QString query = "select ?gid ?name where {"
					"?gid a <" NCO "ContactGroup> ."
					"?gid <" NCO "contactGroupName> ?name }";
	Soprano::Util::AsyncResult* result = d->soprano->executeQueryAsync(query, Soprano::Query::QueryLanguageSparql);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(groups_cb(Soprano::Util::AsyncResult*)));
}

void NepomukProxy::getRegions(const QString& path)
{
	QString query = QString("select ?mid ?uid ?coords ?name where {"
							"?mid a <" NIR "Region> ."
							"?uid a <" NCO "PersonContact> ."
							"<file://%1> <" NIR "region> ?mid . "
							"?mid <" NIR "regionDepicts> ?uid . "
							"?uid <" NCO "fullname> ?name . "
							"?mid <" NIR "coords> ?coords }").arg(QString::fromAscii(QUrl(path).toEncoded()));
	Soprano::Util::AsyncResult* result = d->soprano->executeQueryAsync(query, Soprano::Query::QueryLanguageSparql);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(regions_cb(Soprano::Util::AsyncResult*)));
}

void NepomukProxy::contacts_cb(Soprano::Util::AsyncResult* result)
{
	emit contacts(result->value().value<Soprano::QueryResultIterator>().allBindings());
}

void NepomukProxy::groups_cb(Soprano::Util::AsyncResult* result)
{
	emit groups(result->value().value<Soprano::QueryResultIterator>().allBindings());
}

void NepomukProxy::regions_cb(Soprano::Util::AsyncResult* result)
{
	emit regions(result->value().value<Soprano::QueryResultIterator>().allBindings());
}

void NepomukProxy::addContact_cb(Soprano::Util::AsyncResult*)
{
	d->addContactCompletion --;

	if(!d->addContactCompletion)
		emit contactAdded();
}

void NepomukProxy::addGroup_cb(Soprano::Util::AsyncResult*)
{
	d->addGroupCompletion --;

	if(!d->addGroupCompletion)
		emit groupAdded();
}

void NepomukProxy::addRegion_cb(Soprano::Util::AsyncResult*)
{
	d->addGroupCompletion --;

	// Gory hack to handle the case where several addRegion were running concurrently
	// moreover, it doesn't really mean one of the two queries are finished, it's more of a progress indicator
	if(d->addRegionCompletion%4 == 0)
		emit regionAdded();
}

void NepomukProxy::finishUpdate(Soprano::Util::AsyncResult* oldresult)
{
	d->updateRegionCompletion --;

	if (d->updateRegionCompletion%2 != 0) {
		return;
	}

	d->updateRegionCompletion += 2;

	Soprano::Node predicate;
	Soprano::Node object;
	Soprano::Util::AsyncResult* result;

	QRectF rect = d->queryData[oldresult]["rect"].toRectF();
	QString uid = d->queryData[oldresult]["uid"].toString();
	QString mid = d->queryData[oldresult]["mid"].toString();

	predicate = QUrl::fromEncoded(NIR "coords", QUrl::StrictMode);
	object = Soprano::LiteralValue(QString().sprintf("%f,%f,%f,%f", rect.x(), rect.y(), rect.width(), rect.height()));
	result = d->soprano->addStatementAsync(QUrl(mid), predicate, object);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(updateRegion_cb(Soprano::Util::AsyncResult*)));

	predicate =  QUrl::fromEncoded(NIR "regionDepicts", QUrl::StrictMode);
	object = QUrl(uid);
	result = d->soprano->addStatementAsync(QUrl(mid), predicate, object);
	connect(result, SIGNAL(resultReady(Soprano::Util::AsyncResult*)), this, SLOT(updateRegion_cb(Soprano::Util::AsyncResult*)));

}

void NepomukProxy::updateRegion_cb(Soprano::Util::AsyncResult*)
{
	d->updateRegionCompletion --;

	if(d->updateRegionCompletion%2 == 0)
		emit regionUpdated();
}

void NepomukProxy::removeRegion_cb(Soprano::Util::AsyncResult*)
{
	d->removeRegionCompletion --;

	if(d->updateRegionCompletion%4 == 0)
		emit regionRemoved();
}


NepomukProxy::NepomukProxy() : QObject(NULL), d(new Private)
{
	qDebug() << "Creating the Nepomuk proxy object";
	d->addContactCompletion = 0;
	d->addGroupCompletion = 0;
	d->addRegionCompletion = 0;
	d->updateRegionCompletion = 0;
	d->removeRegionCompletion = 0;
	d->soprano = new Soprano::Util::AsyncModel(Nepomuk::ResourceManager::instance()->mainModel());
}

Q_DECLARE_METATYPE(Soprano::QueryResultIterator);

#include "nepomukproxy.moc"
