/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2008 Adrien Bustany <madcat@mymadcat.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include <QtCore>
#include <klocalizedstring.h>

#include "peoplemodel.h"
#include "nepomukproxy.h"

#include <soprano/soprano.h>

#define TEXT_REFRESHINGDATA i18n("Refreshing data...")

using namespace Nepomuk;

PeopleModel::PeopleModel(QObject *parent) : QAbstractItemModel(parent)
{
	// Set up model
	m_root = new Item;
	// Add the special "not in any group" group
	m_root->children.append(new Item(QString(), TEXT_REFRESHINGDATA));
	
	// Retrieve groups and people
	m_nepomuk = NepomukProxy::getInstance();
	connect(m_nepomuk, SIGNAL(contacts(QList<Soprano::BindingSet>)), this, SLOT(setPeople(QList<Soprano::BindingSet>)));
	emit beginReload();
	m_nepomuk->getContacts();

	// Reload the data when needed
	connect(m_nepomuk, SIGNAL(contactAdded()), this, SLOT(reloadData()));
}

PeopleModel::~PeopleModel()
{
	clearModel();
	delete(m_root);
}

////////// Model related functions //////////

QVariant PeopleModel::data(const QModelIndex &index, int role) const
{
	if (!index.isValid()) return QVariant();

	switch(role) {
		case Qt::DisplayRole:
			return static_cast<Item*>(index.internalPointer())->name;

		default:
			return QVariant();
	}
}

int PeopleModel::rowCount(const QModelIndex &parent) const
{
	if (!parent.isValid()) // Root node
		return m_root->children.size();

	Item *i = static_cast<Item*>(parent.internalPointer());
	return i->children.size();
}

int PeopleModel::columnCount(const QModelIndex &/*parent*/) const
{
	return 1;
}

QModelIndex PeopleModel::index(int row, int column, const QModelIndex &parent) const
{
	if (!parent.isValid()) // The only node which has no parent is the root
		return createIndex(row, 0, static_cast<void*>(m_root->children.at(row)));

	Item *item = static_cast<Item*>(parent.internalPointer());
	if (row >= item->children.size()) return QModelIndex();

	// People
	return createIndex(row, column, static_cast<void*>(item->children.at(row)));
	
}

// Note : I don't even want to think about this function's perfomances...
QModelIndex PeopleModel::parent(const QModelIndex &index) const
{
	if (!index.isValid()) return QModelIndex();
	Item *item = static_cast<Item*>(index.internalPointer());
	if (item == m_root) return QModelIndex();

	// Is it a group ?
	foreach(Item *i, m_root->children) {
		if (i == item) return QModelIndex();
	}

	// Is it a person ?
	foreach(Item *g, m_root->children) {
		for (int i = 0 ; i < g->children.size() ; ++i) {
			if (g->children.at(i) == item)
				return createIndex(i, 0, static_cast<void*>(g));
		}
	}

	return QModelIndex(); // Should never be reached
}

QVariant PeopleModel::headerData(int section, Qt::Orientation orientation, int role) const
{
	Q_UNUSED(section);
	Q_UNUSED(orientation);
	switch(role) {
		case Qt::DisplayRole:
			return i18n("Nepomuk contacts");
		default:
			return QVariant();
	}
}

////////// End of model related functions //////////

void PeopleModel::reloadData()
{
	emit beginReload();
	clearModel();

	// Put the model in a "waiting" state
	m_root = new Item;
	m_root->children.append(new Item(QString(), TEXT_REFRESHINGDATA));

	emit layoutChanged();
	m_nepomuk->getContacts();
}

void PeopleModel::setPeople(QList<Soprano::BindingSet> results)
{
	m_root->children.first()->name = i18n("Not in any group");

	foreach(Soprano::BindingSet set, results) {
		qDebug() << "new person" << set["name"]; // DEBUG
		Item *person = new Item(set["uid"].uri().toString(), set["name"].literal().toString());

		if (set["gid"].isEmpty()) { // The person is not in any group
			m_root->children.at(0)->children.append(person);
			person->parent = m_root->children.at(0);
		} else {
			// Check if the group already exists in our model
			int groupIndex = -1;
			Item* group;
			for (int i = 0 ; i < m_root->children.size() ; ++i)
				if (m_root->children.at(i)->id == set["gid"].uri().toString())
				   groupIndex = i;

			if (groupIndex == -1) { // The group doesn't exist yet in the model
				group = new Item(set["gid"].uri().toString(), set["gname"].literal().toString());
				m_root->children.append(group);
			} else {
				group = m_root->children.at(groupIndex);
			}

			group->children.append(person);
			person->parent = group;
		}
	}

	// DEBUG :Display the model contents
	foreach(Item *i, m_root->children) {
		qDebug() << "group" << i->id << ":" << i->name;
		foreach(Item *j, i->children) {
			qDebug() << "	" << j->name;
		}
	}
	emit layoutChanged();
	emit endReload();
}

// Warning : after calling this function you MUST emit layoutChanged
void PeopleModel::clearModel()
{
	foreach(Item *i, m_root->children) {
		foreach(Item *j, i->children) {
			delete(j);
		}
		delete(i);
	}
}

#include "peoplemodel.moc"
