/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2008 Adrien Bustany <madcat@mymadcat.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include <klocalizedstring.h>

#include "linkedpeoplemodel.h"
#include "item.h"
#include "nepomukproxy.h"

Nepomuk::LinkedPeopleModel::LinkedPeopleModel(QObject *parent) : QAbstractListModel(parent)
{
	m_people.insert(Nepomuk::Item("", i18n("Loading data...")), NULL);
	emit layoutChanged();
}

// Adds a People-Marquee tuple to the model
// We copy the item because the model can be refreshed (and a pointer could become invalid)
void Nepomuk::LinkedPeopleModel::addItem(Item item, Marquee *marquee)  
{
	// If we already have the marquee, remove the corresponding entry (happens when the user reassigns a marquee)
	Nepomuk::Item needle = m_people.key(marquee, Item());
	if (not (needle == Item())) {
		m_people.remove(needle);
	}
	m_people.insert(item, marquee);
	emit layoutChanged(); // Gore, but we can't know the line if we removed an entry
	emit changed();
}

// Removes a People-Marquee tuple (the marquee will get deleted by PeopleTagWidget::removeLinkedPeople
void Nepomuk::LinkedPeopleModel::remove(const QModelIndex &index)
{
	Nepomuk::Item i = m_people.keys().at(index.row());
	beginRemoveRows(QModelIndex(), index.row(), index.row());
	endRemoveRows();
	emit changed();
	if (m_people.value(i)->id() != QString()) { // If the marquee is also stored in Nepomuk, mark it for removal
		m_removedMarquees.append(m_people.value(i)->id());
	}
	m_people.remove(i);
}

// Returns the index linked to the person with the marquee m
QModelIndex Nepomuk::LinkedPeopleModel::indexFromMarquee(Marquee *m)
{
	Nepomuk::Item needle = m_people.key(m, Item());
	if (needle == Item()) return QModelIndex();
	return index(m_people.keys().indexOf(needle), 0, QModelIndex());
}

// Returns the marquee associated with the person at the given index
Nepomuk::Marquee* Nepomuk::LinkedPeopleModel::marqueeFromIndex(const QModelIndex &index)
{
	return m_people.value(m_people.keys().at(index.row()));
}

bool Nepomuk::LinkedPeopleModel::containsId(const QString& id)
{
	if(id == "") return false; // Workaround for the pseudo groups like "Not in any group"
	return m_people.keys().contains(Item(id));
}

int Nepomuk::LinkedPeopleModel::rowCount(const QModelIndex &/*parent*/) const
{
	return m_people.size();
}


QVariant Nepomuk::LinkedPeopleModel::data(const QModelIndex &index, int role) const
{
	switch(role) {
		case Qt::DisplayRole:
			if (index.row() >= m_people.size()) return QVariant();
			return m_people.keys().at(index.row()).name;
		default:
			return QVariant();
	}
}

void Nepomuk::LinkedPeopleModel::sync(const QString& filepath)
{
	qDebug() << "entree dans sync";
	m_pendingOperations = m_people.keys().size() + m_removedMarquees.size();

	Nepomuk::NepomukProxy* nepomuk = Nepomuk::NepomukProxy::getInstance();
	connect(nepomuk, SIGNAL(regionAdded()), this, SLOT(sync_cb()));
	connect(nepomuk, SIGNAL(regionUpdated()), this, SLOT(sync_cb()));
	connect(nepomuk, SIGNAL(regionRemoved()), this, SLOT(sync_cb()));

	Nepomuk::Marquee* m;
	foreach (Nepomuk::Item i, m_people.keys()) {
		qDebug() << "processing region (add/update)";
		m = m_people.value(i);
		if (m->id() == QString()) { // This is a new marquee
			nepomuk->addRegion(filepath, m->toRectF(), i.id);
		} else {
			nepomuk->updateRegion(m->id(), m->toRectF(), i.id);
		}
	}

	foreach (QString uid, m_removedMarquees) {
		qDebug() << "deleting region";
		nepomuk->removeRegion(uid);
	}
	qDebug() << "sortie de sync";
}

// Clears the model and displays a loading message
void Nepomuk::LinkedPeopleModel::setLoading(bool loading)
{
	foreach (Nepomuk::Item i, m_people.keys())
		m_people.remove(i);

	if (loading) {
		m_people.insert(Nepomuk::Item("", i18n("Loading data...")), NULL);
	}
	emit layoutChanged();
}

void Nepomuk::LinkedPeopleModel::sync_cb()
{
	m_pendingOperations--;

	if(!m_pendingOperations)
		emit synced();
}

#include "linkedpeoplemodel.moc"
