/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2008 Adrien Bustany <madcat@mymadcat.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef LINKEDPEOPLEMODEL_H
#define LINKEDPEOPLEMODEL_H

#include <QtCore/QAbstractListModel>
#include <nepomuk/resource.h>
#include <nepomuk/resourcemanager.h> // Needed for uri generation

#include "item.h"
#include "marquee.h"

namespace Nepomuk {

/**
 * \class LinkedPeopleModel linkedpeoplemodel.h LinkedPeopleModel
 *
 * \brief Model used for the list of "linked" people, i.e. people associated to a zone (marquee)
 * on the picture.
 *
 * This model links people (reprensented as a [resource id | fullname] tuple) to their marquee.
 * When created, it also load the existing marquees from Nepomuk.
 *
 * \author Adrien Bustany <madcat@mymadcat.com>
 */

class LinkedPeopleModel : public QAbstractListModel
{
	Q_OBJECT

public:
				LinkedPeopleModel(QObject *parent = 0);
	void		addItem(Item item, Marquee *marquee);
	void		remove(const QModelIndex& index);
	QModelIndex indexFromMarquee(Marquee *m);
	Marquee*	marqueeFromIndex(const QModelIndex& index);
	/**
	 * Tests if the model contains a person with the given resource id.
	 * This is used by the %FilterModel to mask already linked persons.
	 */
	bool		containsId(const QString& id);

	int			rowCount(const QModelIndex& parent = QModelIndex()) const;
	QVariant	data(const QModelIndex& index, int role = Qt::DisplayRole) const;

public Q_SLOTS:
	/**
	 * Syncs the model to Nepomuk (one way sync)
	 * \param filename The filed linked to the model
	 */
	void sync(const QString& filepath);

	void setLoading(bool loading);

signals:
	/**
	 * Used by PeopleModel to know when to recompute the people to display, as LinkedPeopleModel is a "mask" model
	 */
	void changed();

	/**
	 * Emitted when sync is done
	 */
	void synced();

private Q_SLOTS:
	void sync_cb();

private:
	QHash<Item, Marquee*>	m_people;
	void					clearModel();
	unsigned int			m_pendingOperations;
	QList<QString>			m_removedMarquees; // Stores the marquees to remove when saving
};

} // End namespace Nepomuk

#endif
