/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2008 Adrien Bustany <madcat@mymadcat.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include <kiconloader.h>

#include "ui_widget.h"

#include "peopletagwidget.h"
#include "imagemarquee.h"
#include "peoplemodel.h"
#include "linkedpeoplemodel.h"
#include "otherpersondialog.h"
#include "filtermodel.h"
#include "item.h"
#include "nepomukproxy.h"
#include "marquee.h"

Nepomuk::PeopleTagWidget::PeopleTagWidget(QWidget *parent) : QWidget(parent)
{
	m_gui = new Ui::Interface();
	m_gui->setupUi(this);
	// Load the pixmaps of the ui with KDE's default icons
	m_gui->zoomInIcon->setPixmap(DesktopIcon("zoom-in", 32));
	m_gui->zoomOutIcon->setPixmap(DesktopIcon("zoom-out", 32));

	// Zoom slider set up
	connect(m_gui->zoomSlider, SIGNAL(valueChanged(int)), this, SLOT(zoomScene()));

	// View and scene setup
	m_imageScene = new Nepomuk::ImageMarquee(this);
	m_gui->imageView->setScene(m_imageScene);
	m_gui->imageView->setBackgroundBrush(QBrush(Qt::black));
	connect(m_imageScene, SIGNAL(newMarquee(Marquee*)), this, SLOT(newMarquee(Marquee*)));
	connect(m_imageScene, SIGNAL(marqueeSelected(Marquee*)), this, SLOT(marqueeSelected(Marquee*)));

	// Available people set up (list on the right)
	m_peopleModel = new Nepomuk::PeopleModel(this);
	connect(m_peopleModel, SIGNAL(beginReload()), this, SLOT(modelReloading()));
	connect(m_peopleModel, SIGNAL(endReload()), this, SLOT(modelReloaded()));

	// Linked people set up (list on the bottom)
	m_linkedPeopleModel = new Nepomuk::LinkedPeopleModel(this);
	connect(m_linkedPeopleModel, SIGNAL(synced()), this, SLOT(nepomukSynced()));
	m_gui->addedPeople->setModel(m_linkedPeopleModel);
	m_addedPeopleSelection = m_gui->addedPeople->selectionModel();
	connect(m_addedPeopleSelection, SIGNAL(selectionChanged(const QItemSelection&, const QItemSelection&)), this, SLOT(addedPeopleSelected(const QItemSelection&)));

	// Filter set up
	m_filterModel = new Nepomuk::FilterModel(this);
	m_filterModel->setSourceModel(m_peopleModel);
	m_filterModel->setMaskModel(m_linkedPeopleModel); // Hide the already linked people from the list
	m_filterModel->setDynamicSortFilter(true);
	m_filterModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
	connect(m_gui->searchField, SIGNAL(textChanged(const QString&)), m_filterModel, SLOT(setFilterRegExp(const QString&)));
	m_gui->peopleList->setModel(m_filterModel);

	// Selection model set up
	m_peopleSelection = m_gui->peopleList->selectionModel();
	connect(m_peopleSelection, SIGNAL(selectionChanged(const QItemSelection&, const QItemSelection&)), this, SLOT(peopleSelected(const QItemSelection&)));

	connect(m_gui->removePerson, SIGNAL(clicked()), this, SLOT(removeLinkedPeople()));

	// "Add another person" dialog set up
	m_otherPersonDialog = new Nepomuk::OtherPersonDialog(this);
	connect(m_gui->otherPersonButton, SIGNAL(clicked()), m_otherPersonDialog, SLOT(show()));

	// Get a reference to the Nepomuk singleton
	m_nepomuk = Nepomuk::NepomukProxy::getInstance();
	connect(m_nepomuk, SIGNAL(regions(const QList<Soprano::BindingSet>&)), this, SLOT(addRegions(const QList<Soprano::BindingSet>&)));
}

void Nepomuk::PeopleTagWidget::loadPicture(const QString &filename)
{
	// We need the absolute path to identify the file with Nepomuk
	QFileInfo fileInfo(filename);
	m_picturePath = fileInfo.absoluteFilePath();
	m_imageScene->loadPicture(m_picturePath);
	m_gui->imageView->fitInView(m_imageScene->sceneRect(), Qt::KeepAspectRatio);
	originalMatrix = m_gui->imageView->matrix();

	// Load exising regions from Nepomuk
	m_nepomuk->getRegions(m_picturePath);
	m_gui->imageView->setEnabled(false);
	m_gui->addedPeople->setEnabled(false);
	m_linkedPeopleModel->setLoading(true);
}

// Invoked by PictureFrame when the user releases the mouse after having drawn a marquee
void Nepomuk::PeopleTagWidget::newMarquee(Marquee *m)
{
	m_currentMarquee = m;

	// If there's already somebody selected in the people list, link it to the marquee
	if (m_peopleSelection->selectedIndexes().size() == 1) {
		peopleSelected(m_peopleSelection->selection());
	}

	// Clear the selections, it can get confusing for the user else (new marquee, and someone already selected)
	m_peopleSelection->clear();
	m_addedPeopleSelection->clear();

	// Highlight the entry corresponding to the marquee in the "linked people" list
	marqueeSelected(m);
}

// Called when the user clicks on an existing marquee
void Nepomuk::PeopleTagWidget::marqueeSelected(Marquee *m)
{
	QModelIndex selectedPerson = m_linkedPeopleModel->indexFromMarquee(m);

	// Highlight the corresponding entry in the "linked people" list if it exists
	if (selectedPerson != QModelIndex()) {
		m_addedPeopleSelection->clear();
		m_addedPeopleSelection->select(selectedPerson, QItemSelectionModel::Select);
	}
	m_gui->removePerson->setEnabled(true);
}

// Called when the user selects a person in the people list
// We then link the currently selected marquee with the selected person
void Nepomuk::PeopleTagWidget::peopleSelected(const QItemSelection &item)
{
	if (!m_peopleSelection->hasSelection()) return; // We don't have any selection when we clear the selection
	QModelIndex index = m_filterModel->mapToSource(item.indexes().first());
	if (index.parent() == QModelIndex())
		return; // Only take the selection of persons into account

	Nepomuk::Item *person = static_cast<Item*>(index.internalPointer());
	if (m_imageScene->selectedItems().isEmpty()) return; // This should never happen (ie no selection)
	Nepomuk::Marquee *m = static_cast<Marquee*>(m_imageScene->selectedItems().first()->parentItem());
	if (!m) m = static_cast<Marquee*>(m_imageScene->selectedItems().first()); // A bit hackish way to detect if we have a group

	m_linkedPeopleModel->addItem(*person, m); // We copy the person since it might change during a model refresh
	m->setLabel(person->name);
	marqueeSelected(m);
	zoomScene();
}

// Called when the user selects a person in the "linked people" list
// We then select the corresponding marquee in the view
void Nepomuk::PeopleTagWidget::addedPeopleSelected(const QItemSelection &item)
{
	if (item.indexes().isEmpty()) return;
	
	Nepomuk::Marquee *m = m_linkedPeopleModel->marqueeFromIndex(item.indexes().first());
	m_imageScene->deselectOtherMarquees(m);
	m->setSelected(true);
	m_gui->removePerson->setEnabled(true);
}

// Unlink a person from the picture, and remove the corresponding marquee
void Nepomuk::PeopleTagWidget::removeLinkedPeople()
{
	if (m_addedPeopleSelection->selectedIndexes().isEmpty()) {
		return; // Should never happen, but who knows ?
	}

	QModelIndex selectedIndex = m_addedPeopleSelection->selectedIndexes().first();
	Nepomuk::Marquee *m = m_linkedPeopleModel->marqueeFromIndex(selectedIndex);

	m_linkedPeopleModel->remove(selectedIndex);
	m->deleteLater();
}

void Nepomuk::PeopleTagWidget::modelReloading()
{
	m_gui->imageView->setEnabled(false);
	m_gui->searchField->setEnabled(false);
}

void Nepomuk::PeopleTagWidget::modelReloaded()
{
	m_gui->imageView->setEnabled(true);
	m_gui->searchField->setEnabled(true);
}

void Nepomuk::PeopleTagWidget::zoomScene()
{
	int value = m_gui->zoomSlider->value();
	QMatrix matrix = originalMatrix;
	matrix.scale(1+value/100., 1+value/100.);
	m_gui->imageView->setMatrix(matrix);
	m_imageScene->matrixChanged(matrix);
}

// Callback for the regions signal of the Nepomuk proxy
void Nepomuk::PeopleTagWidget::addRegions(const QList<Soprano::BindingSet>& regions)
{
	m_gui->imageView->setEnabled(true);
	m_linkedPeopleModel->setLoading(false);
	QStringList coords;
	qreal x, y, w, h;
	foreach (Soprano::BindingSet set, regions) {
		coords = set["coords"].literal().toString().split(",", QString::SkipEmptyParts);
		x = coords[0].toDouble();
		y = coords[1].toDouble();
		w = coords[2].toDouble();
		h = coords[3].toDouble();
		qDebug() << "new region" << x << y << w << h;
		Marquee *m = m_imageScene->addExistingMarquee(x, y, w, h, set["mid"].uri().toString());
		m->setLabel(set["name"].literal().toString());
		m_linkedPeopleModel->addItem(Item(set["uid"].uri().toString(), set["name"].literal().toString()), m);
	}

	m_gui->addedPeople->setEnabled(true);
	m_imageScene->deselectOtherMarquees(NULL);
	zoomScene(); // This one is only to resize the labels on the marquees
}

void Nepomuk::PeopleTagWidget::save()
{
	m_gui->imageView->setEnabled(false);
	qDebug() << "appel de sync depuis peopletagwidget";
	m_linkedPeopleModel->sync(m_picturePath);
}

void Nepomuk::PeopleTagWidget::nepomukSynced()
{
	m_gui->imageView->setEnabled(true);
	emit saved();
}

void Nepomuk::PeopleTagWidget::showEvent(QShowEvent* event)
{
	m_gui->imageView->fitInView(m_imageScene->sceneRect(), Qt::KeepAspectRatio);
	QWidget::showEvent(event);
	originalMatrix = m_gui->imageView->matrix();
	zoomScene();
}

void Nepomuk::PeopleTagWidget::resizeEvent(QResizeEvent* event)
{
	m_gui->imageView->fitInView(m_imageScene->sceneRect(), Qt::KeepAspectRatio);
	QWidget::resizeEvent(event);
	originalMatrix = m_gui->imageView->matrix();
	zoomScene();
}

#include "peopletagwidget.moc"
