/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 */

#include "nepomukcontactresourceguiitem.h"

#include "nco.h"

#include <KIcon>
#include <KPluginFactory>
#include <KPluginLoader>
#include <KDebug>

#include <nepomuk/resource.h>
#include <nepomuk/variant.h>

#ifdef HAVE_AKONADI
#include <akonadi/item.h>
#include <akonadi/itemfetchjob.h>
#include <akonadi/itemfetchscope.h>
#include <kabc/addressee.h>
#endif

K_PLUGIN_FACTORY(ContactResourceGuiItemFactory, registerPlugin<Nepomuk::ContactResourceGuiItem>();)
K_EXPORT_PLUGIN(ContactResourceGuiItemFactory("nepomuk_contactresourceguiitem"))


class Nepomuk::ContactResourceGuiItem::Private
{
public:
    Private( ContactResourceGuiItem* parent )
        : q( parent ) {
    }

#ifdef HAVE_AKONADI
    const KABC::Addressee addresseeForUri( const QUrl& uri );

    QHash<int, KABC::Addressee> addresseeCache;
#endif
private:
    ContactResourceGuiItem* q;
};


#ifdef HAVE_AKONADI
const KABC::Addressee Nepomuk::ContactResourceGuiItem::Private::addresseeForUri( const QUrl& uri )
{
    Akonadi::Item item = Akonadi::Item::fromUrl( uri );
    QHash<int, KABC::Addressee>::const_iterator it = addresseeCache.find( item.id() );
    if ( it != addresseeCache.constEnd() ) {
        return *it;
    }
    else {
        Akonadi::ItemFetchJob* job = new Akonadi::ItemFetchJob( item );
        connect( job, SIGNAL( result( KJob* ) ), q, SLOT( slotJobFinished( KJob* ) ) );
        job->fetchScope().fetchFullPayload();
        job->start();
        addresseeCache.insert( item.id(), KABC::Addressee() );
        return KABC::Addressee();
    }
}
#endif


Nepomuk::ContactResourceGuiItem::ContactResourceGuiItem( QObject* parent, const QList<QVariant>& )
    : GenericResourceGuiItem( parent ),
      d( new Private(this) )
{
}


Nepomuk::ContactResourceGuiItem::~ContactResourceGuiItem()
{
    delete d;
}


QString Nepomuk::ContactResourceGuiItem::genericLabel( const Resource& resource )
{
    QString s;
#ifdef HAVE_AKONADI
    s = d->addresseeForUri( resource.resourceUri() ).formattedName();
#endif
    if ( s.isEmpty() )
        s = resource.property( Nepomuk::Vocabulary::NCO::fullname() ).toString();

    return s.isEmpty() ? resource.genericLabel() : s;
}


QString Nepomuk::ContactResourceGuiItem::genericDescription( const Resource& resource )
{
#ifdef HAVE_AKONADI
    QString email = d->addresseeForUri( resource.resourceUri() ).preferredEmail();
    if ( !email.isEmpty() )
        return email;
#endif
    QStringList s;
    QList<Resource> emailAddresses = resource.property( Nepomuk::Vocabulary::NCO::hasEmailAddress() ).toResourceList();
    for ( int i = 0; i < emailAddresses.count(); ++i ) {
        s += emailAddresses[i].property( Vocabulary::NCO::emailAddress() ).toString();
    }
    return s.join( "\n" );
}


QPixmap Nepomuk::ContactResourceGuiItem::genericPixmap( const Resource& resource, const QSize& size )
{
#ifdef HAVE_AKONADI
    KABC::Picture photo = d->addresseeForUri( resource.resourceUri() ).photo();
    if ( !photo.isEmpty() && photo.isIntern() ) {
        return QPixmap::fromImage( photo.data().scaled( size ) );
    }
#endif
    // this will at least pull in nao:hasSymbol
    QString icon = resource.genericIcon();
    if ( !icon.isEmpty() ) {
        return KIcon( icon ).pixmap( size );
    }
    return KIcon( "personal" ).pixmap( size );
}


#ifdef HAVE_AKONADI
void Nepomuk::ContactResourceGuiItem::slotJobFinished( KJob* job )
{
    if ( !job->error() ) {
        Akonadi::ItemFetchJob* akonadiJob = static_cast<Akonadi::ItemFetchJob*>( job );
        Akonadi::Item item = akonadiJob->items().first();
        d->addresseeCache[item.id()] = item.payload<KABC::Addressee>();
        emit dataChanged( item.url() );
    }
}
#endif

#include "nepomukcontactresourceguiitem.moc"
