/*
 * This file is part of Soprano Project.
 *
 * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _SOPRANO_SWRC_H_
#define _SOPRANO_SWRC_H_

#include <QtCore/QUrl>

namespace Nepomuk {
    namespace Vocabulary {
        namespace Ontoware {
            namespace SWRC {
                /**
                 * http://purl.org/dc/elements/1.1/
                 */
                QUrl swrcNamespace();

                /**
                 * http://purl.org/dc/elements/1.1/contributor 
                 * 
                 * check range. Olivier 2006-06-07 
                 * Philipp: checked. 
                 */
                QUrl contributor();

                /**
                 * http://purl.org/dc/elements/1.1/coverage 
                 * 
                 * Definition: The extent or scope of the content of the resource. 
                 * Comment: Typically, Coverage will include spatial location 
                 * (a place name or geographic coordinates), temporal period 
                 * (a period label, date, or date range) or jurisdiction (such 
                 * as a named administrative entity). Recommended best practice 
                 * is to select a value from a controlled vocabulary (for example, 
                 * the Thesaurus of Geographic Names [TGN]) and to use, where appropriate, 
                 * named places or time periods in preference to numeric identifiers 
                 * such as sets of coordinates or date ranges. 
                 */
                QUrl coverage();

                /**
                 * http://purl.org/dc/elements/1.1/creator 
                 * 
                 * Definition: An entity primarily responsible for making the 
                 * content of the resource (for instance an author). 
                 * Comment: Examples of Creator include a person, an organization, 
                 * or a service. Typically, the name of a Creator should be used 
                 * to indicate the entity. 
                 */
                QUrl creator();

                /**
                 * http://purl.org/dc/elements/1.1/date 
                 * 
                 * Definition: A date of an event in the lifecycle of the resource. 
                 * 
                 * Comment: Typically, Date will be associated with the creation 
                 * or availability of the resource. Recommended best practice 
                 * for encoding the date value is defined in a profile of ISO 8601 
                 * [W3CDTF] and includes (among others) dates of the form YYYY-MM-DD. 
                 */
                QUrl date();

                /**
                 * http://purl.org/dc/elements/1.1/description 
                 * 
                 * Definition: An account of the content of the resource. 
                 * Comment: Examples of Description include, but is not limited 
                 * to: an abstract, table of contents, reference to a graphical 
                 * representation of content or a free-text account of the content. 
                 */
                QUrl description();

                /**
                 * http://purl.org/dc/elements/1.1/format 
                 * 
                 * Definition: The physical or digital manifestation of the resource. 
                 * Comment: Typically, Format may include the media-type or dimensions 
                 * of the resource. Format may be used to identify the software, 
                 * hardware, or other equipment needed to display or operate the 
                 * resource. Examples of dimensions include size and duration. 
                 * Recommended best practice is to select a value from a controlled 
                 * vocabulary (for example, the list of Internet Media Types [MIME] 
                 * defining computer media formats). 
                 */
                QUrl format();

                /**
                 * http://purl.org/dc/elements/1.1/identifier 
                 * 
                 * Definition: An unambiguous reference to the resource within 
                 * a given context. 
                 * Comment: Recommended best practice is to identify the resource 
                 * by means of a string or number conforming to a formal identification 
                 * system. Formal identification systems include but are not 
                 * limited to the Uniform Resource Identifier (URI) (including 
                 * the Uniform Resource Locator (URL)), the Digital Object Identifier 
                 * (DOI) and the International Standard Book Number (ISBN). 
                 */
                QUrl identifier();

                /**
                 * http://purl.org/dc/elements/1.1/language 
                 * 
                 * Definition: A language of the intellectual content of the resource. 
                 * Comment: Recommended best practice is to use RFC 3066 [RFC3066] 
                 * which, in conjunction with ISO639 [ISO639]), defines two- 
                 * and three-letter primary language tags with optional subtags. 
                 * Examples include "en" or "eng" for English, "akk" for Akkadian", 
                 * and "en-GB" for English used in the United Kingdom. 
                 */
                QUrl language();

                /**
                 * http://purl.org/dc/elements/1.1/publisher 
                 * 
                 * check domain. Olivier 2006-06-07 
                 * Philipp: checked. 
                 */
                QUrl publisher();

                /**
                 * http://purl.org/dc/elements/1.1/relation 
                 * 
                 * Definition: A reference to a related resource. 
                 * Comment: Recommended best practice is to identify the referenced 
                 * resource by means of a string or number conforming to a formal 
                 * identification system. 
                 */
                QUrl relation();

                /**
                 * http://purl.org/dc/elements/1.1/rights 
                 * 
                 * Definition: Information about rights held in and over the resource. 
                 * Comment: Typically, Rights will contain a rights management 
                 * statement for the resource, or reference a service providing 
                 * such information. Rights information often encompasses Intellectual 
                 * Property Rights (IPR), Copyright, and various Property Rights. 
                 * If the Rights element is absent, no assumptions may be made about 
                 * any rights held in or over the resource. 
                 */
                QUrl rights();

                /**
                 * http://purl.org/dc/elements/1.1/source 
                 * 
                 * Definition: A Reference to a resource from which the present 
                 * resource is derived. 
                 * Comment: The present resource may be derived from the Source 
                 * resource in whole or in part. Recommended best practice is to 
                 * identify the referenced resource by means of a string or number 
                 * conforming to a formal identification system. 
                 */
                QUrl source();

                /**
                 * http://purl.org/dc/elements/1.1/subject 
                 * 
                 * Definition: A topic of the content of the resource. 
                 * Comment: Typically, Subject will be expressed as keywords, 
                 * key phrases or classification codes that describe a topic of 
                 * the resource. Recommended best practice is to select a value 
                 * from a controlled vocabulary or formal classification scheme. 
                 */
                QUrl subject();

                /**
                 * http://purl.org/dc/elements/1.1/title 
                 * 
                 * swrc:booktitle removed, since it is redundant to this property 
                 * here. 
                 */
                QUrl title();

                /**
                 * http://purl.org/dc/elements/1.1/type 
                 * 
                 * Definition: The nature or genre of the content of the resource. 
                 * Comment: Type includes terms describing general categories, 
                 * functions, genres, or aggregation levels for content. Recommended 
                 * best practice is to select a value from a controlled vocabulary 
                 * (for example, the DCMI Type Vocabulary [DCT1]). To describe 
                 * the physical or digital manifestation of the resource, use 
                 * the FORMAT element. 
                 */
                QUrl type();

                /**
                 * http://swrc.ontoware.org/ontology#AcademicStaff 
                 */
                QUrl AcademicStaff();

                /**
                 * http://swrc.ontoware.org/ontology#AdministrativeStaff 
                 */
                QUrl AdministrativeStaff();

                /**
                 * http://swrc.ontoware.org/ontology#Article 
                 * 
                 * A peer reviewed , refereed article from a journal. 
                 */
                QUrl Article();

                /**
                 * http://swrc.ontoware.org/ontology#AssistantProfessor 
                 */
                QUrl AssistantProfessor();

                /**
                 * http://swrc.ontoware.org/ontology#AssociateProfessor 
                 */
                QUrl AssociateProfessor();

                /**
                 * http://swrc.ontoware.org/ontology#Association 
                 */
                QUrl Association();

                /**
                 * http://swrc.ontoware.org/ontology#Book 
                 * 
                 * A complete book, not formed from separate papers. 
                 */
                QUrl Book();

                /**
                 * http://swrc.ontoware.org/ontology#Booklet 
                 * 
                 * A work that is printed and bound but without a named publisher 
                 * or sponsering institution. 
                 */
                QUrl Booklet();

                /**
                 * http://swrc.ontoware.org/ontology#Collection 
                 * 
                 * A book produced from a collection of separate papers. 
                 */
                QUrl Collection();

                /**
                 * http://swrc.ontoware.org/ontology#Colloquium 
                 */
                QUrl Colloquium();

                /**
                 * http://swrc.ontoware.org/ontology#Conference 
                 */
                QUrl Conference();

                /**
                 * http://swrc.ontoware.org/ontology#Department 
                 */
                QUrl Department();

                /**
                 * http://swrc.ontoware.org/ontology#DevelopmentProject 
                 * 
                 * disjoint with researchProject? maybe not. Olivier 2006-06-07 
                 */
                QUrl DevelopmentProject();

                /**
                 * http://swrc.ontoware.org/ontology#DiplomaThesis 
                 */
                QUrl DiplomaThesis();

                /**
                 * http://swrc.ontoware.org/ontology#Document 
                 * 
                 * Dublin Core elements added. 
                 * See: http://dublincore.org/documents/dces/ 
                 * 
                 * for document types see: 
                 * http://www.bsz-bw.de/diglib/medserv/konvent/metadat/dcresour.html 
                 */
                QUrl Document();

                /**
                 * http://swrc.ontoware.org/ontology#Employee 
                 */
                QUrl Employee();

                /**
                 * http://swrc.ontoware.org/ontology#Enterprise 
                 */
                QUrl Enterprise();

                /**
                 * http://swrc.ontoware.org/ontology#Event 
                 */
                QUrl Event();

                /**
                 * http://swrc.ontoware.org/ontology#ExchangeProfessor 
                 */
                QUrl ExchangeProfessor();

                /**
                 * http://swrc.ontoware.org/ontology#Exhibition 
                 */
                QUrl Exhibition();

                /**
                 * http://swrc.ontoware.org/ontology#FacultyMember 
                 */
                QUrl FacultyMember();

                /**
                 * http://swrc.ontoware.org/ontology#FullProfessor 
                 */
                QUrl FullProfessor();

                /**
                 * http://swrc.ontoware.org/ontology#Graduate 
                 * 
                 * disjoint with undergrad? Olivier 2006-06-07 
                 * Philipp: I wouldn't make it disjoint, since someone can be a 
                 * bachelor-graduate, but if he is doing his master he is a master-undergraduate 
                 * at the same time. 
                 */
                QUrl Graduate();

                /**
                 * http://swrc.ontoware.org/ontology#GraduateClass 
                 * 
                 * see: http://www.neurolabor.de/socrate/ects_ganz.htm 
                 */
                QUrl GraduateClass();

                /**
                 * http://swrc.ontoware.org/ontology#HonoraryProfessor 
                 */
                QUrl HonoraryProfessor();

                /**
                 * http://swrc.ontoware.org/ontology#InBook 
                 * 
                 * A part of a book, which may be a chapter and/or range of pages. 
                 */
                QUrl InBook();

                /**
                 * http://swrc.ontoware.org/ontology#InCollection 
                 * 
                 * A single paper or article from a published collection. 
                 */
                QUrl InCollection();

                /**
                 * http://swrc.ontoware.org/ontology#InProceedings 
                 * 
                 * A single paper from a published workshop or conference proceedings. 
                 */
                QUrl InProceedings();

                /**
                 * http://swrc.ontoware.org/ontology#Institute 
                 */
                QUrl Institute();

                /**
                 * http://swrc.ontoware.org/ontology#Journal 
                 * 
                 * An entire issue of a refereed learned journal. 
                 */
                QUrl Journal();

                /**
                 * http://swrc.ontoware.org/ontology#Lecture 
                 */
                QUrl Lecture();

                /**
                 * http://swrc.ontoware.org/ontology#Lecturer 
                 */
                QUrl Lecturer();

                /**
                 * http://swrc.ontoware.org/ontology#Magazine 
                 * 
                 * An entire issue of an unrefereed journal or magazine. 
                 */
                QUrl Magazine();

                /**
                 * http://swrc.ontoware.org/ontology#Manager 
                 */
                QUrl Manager();

                /**
                 * http://swrc.ontoware.org/ontology#Manual 
                 * 
                 * An operations manual for a product. 
                 */
                QUrl Manual();

                /**
                 * http://swrc.ontoware.org/ontology#MasterThesis 
                 * 
                 * A university Masters thesis. 
                 */
                QUrl MasterThesis();

                /**
                 * http://swrc.ontoware.org/ontology#Meeting 
                 */
                QUrl Meeting();

                /**
                 * http://swrc.ontoware.org/ontology#Misc 
                 * 
                 * Work of another or undetermined type. This is the default scheme 
                 * value if the scheme is not explicitly stated. 
                 */
                QUrl Misc();

                /**
                 * http://swrc.ontoware.org/ontology#Organization 
                 */
                QUrl Organization();

                /**
                 * http://swrc.ontoware.org/ontology#Person 
                 */
                QUrl Person();

                /**
                 * http://swrc.ontoware.org/ontology#PhDStudent 
                 */
                QUrl PhDStudent();

                /**
                 * http://swrc.ontoware.org/ontology#PhDThesis 
                 * 
                 * A university Doctoral thesis. 
                 */
                QUrl PhDThesis();

                /**
                 * http://swrc.ontoware.org/ontology#Proceedings 
                 * 
                 * A whole published workshop or conference proceedings. 
                 */
                QUrl Proceedings();

                /**
                 * http://swrc.ontoware.org/ontology#Product 
                 */
                QUrl Product();

                /**
                 * http://swrc.ontoware.org/ontology#ProfessorEmeritus 
                 */
                QUrl ProfessorEmeritus();

                /**
                 * http://swrc.ontoware.org/ontology#Project 
                 */
                QUrl Project();

                /**
                 * http://swrc.ontoware.org/ontology#ProjectMeeting 
                 */
                QUrl ProjectMeeting();

                /**
                 * http://swrc.ontoware.org/ontology#ProjectReport 
                 */
                QUrl ProjectReport();

                /**
                 * http://swrc.ontoware.org/ontology#Publication 
                 */
                QUrl Publication();

                /**
                 * http://swrc.ontoware.org/ontology#Report 
                 */
                QUrl Report();

                /**
                 * http://swrc.ontoware.org/ontology#ResearchGroup 
                 */
                QUrl ResearchGroup();

                /**
                 * http://swrc.ontoware.org/ontology#ResearchPaper 
                 * 
                 * A piece of research work. 
                 */
                QUrl ResearchPaper();

                /**
                 * http://swrc.ontoware.org/ontology#ResearchProject 
                 */
                QUrl ResearchProject();

                /**
                 * http://swrc.ontoware.org/ontology#ResearchTopic 
                 */
                QUrl ResearchTopic();

                /**
                 * http://swrc.ontoware.org/ontology#Root 
                 * 
                 * meaning not clear. Olivier 2006-06-07 
                 * Philipp: Hmm, I didn't create that property, therefore i don't 
                 * know the intention of it. 
                 * 
                 */
                QUrl Root();

                /**
                 * http://swrc.ontoware.org/ontology#RootRelation 
                 * 
                 * meaning not clear. Olivier 2006-06-07 
                 * Philipp: Hmm, I didn't create that property, therefore i don't 
                 * know the intention of it. 
                 * 
                 */
                QUrl RootRelation();

                /**
                 * http://swrc.ontoware.org/ontology#Seminar 
                 */
                QUrl Seminar();

                /**
                 * http://swrc.ontoware.org/ontology#SoftwareComponent 
                 */
                QUrl SoftwareComponent();

                /**
                 * http://swrc.ontoware.org/ontology#SoftwareProject 
                 */
                QUrl SoftwareProject();

                /**
                 * http://swrc.ontoware.org/ontology#Student 
                 */
                QUrl Student();

                /**
                 * http://swrc.ontoware.org/ontology#TechnicalReport 
                 */
                QUrl TechnicalReport();

                /**
                 * http://swrc.ontoware.org/ontology#TechnicalStaff 
                 */
                QUrl TechnicalStaff();

                /**
                 * http://swrc.ontoware.org/ontology#Thesis 
                 */
                QUrl Thesis();

                /**
                 * http://swrc.ontoware.org/ontology#Topic 
                 * 
                 * If changing this concept (or its subconcepts) consider that 
                 * the swrc topic ontology depends on the foresaid one. 
                 * The swrc topic ontology can be found at: 
                 * http://ontoware.org/frs/download.php/187/swrc-swtopics.owl 
                 */
                QUrl Topic();

                /**
                 * http://swrc.ontoware.org/ontology#Undergraduate 
                 */
                QUrl Undergraduate();

                /**
                 * http://swrc.ontoware.org/ontology#UndergraduateAdvancedClass 
                 * 
                 * see: http://www.neurolabor.de/socrate/ects_ganz.htm 
                 */
                QUrl UndergraduateAdvancedClass();

                /**
                 * http://swrc.ontoware.org/ontology#UndergraduateIntermediateClass 
                 * 
                 * see: http://www.neurolabor.de/socrate/ects_ganz.htm 
                 * 
                 * http://dict.leo.org/cgi-bin/dict/urlexp/20030705201638 
                 */
                QUrl UndergraduateIntermediateClass();

                /**
                 * http://swrc.ontoware.org/ontology#University 
                 */
                QUrl University();

                /**
                 * http://swrc.ontoware.org/ontology#Unpublished 
                 * 
                 * A document with an author and title, but not formally published. 
                 */
                QUrl Unpublished();

                /**
                 * http://swrc.ontoware.org/ontology#UnrefereedArticle 
                 * 
                 * An unrefereed article from a journal, magazine or newspaper. 
                 * 
                 * possible german translation for "unrefereed", also: unredigiert 
                 * ?! 
                 */
                QUrl UnrefereedArticle();

                /**
                 * http://swrc.ontoware.org/ontology#VisitingProfessor 
                 */
                QUrl VisitingProfessor();

                /**
                 * http://swrc.ontoware.org/ontology#Workshop 
                 */
                QUrl Workshop();

                /**
                 * http://swrc.ontoware.org/ontology#abstract 
                 */
                QUrl abstract();

                /**
                 * http://swrc.ontoware.org/ontology#address 
                 */
                QUrl address();

                /**
                 * http://swrc.ontoware.org/ontology#affiliation 
                 */
                QUrl affiliation();

                /**
                 * http://swrc.ontoware.org/ontology#atEvent 
                 */
                QUrl atEvent();

                /**
                 * http://swrc.ontoware.org/ontology#carriedOutBy 
                 */
                QUrl carriedOutBy();

                /**
                 * http://swrc.ontoware.org/ontology#carriesOut 
                 */
                QUrl carriesOut();

                /**
                 * http://swrc.ontoware.org/ontology#chapter 
                 */
                QUrl chapter();

                /**
                 * http://swrc.ontoware.org/ontology#cite 
                 * 
                 * check domain. Olivier 2006-06-07 
                 * Philipp: checked. 
                 */
                QUrl cite();

                /**
                 * http://swrc.ontoware.org/ontology#citedBy 
                 */
                QUrl citedBy();

                /**
                 * http://swrc.ontoware.org/ontology#cooperateWith 
                 */
                QUrl cooperateWith();

                /**
                 * http://swrc.ontoware.org/ontology#creationDate 
                 */
                QUrl creationDate();

                /**
                 * http://swrc.ontoware.org/ontology#dealtWithIn 
                 */
                QUrl dealtWithIn();

                /**
                 * http://swrc.ontoware.org/ontology#describesProject 
                 */
                QUrl describesProject();

                /**
                 * http://swrc.ontoware.org/ontology#developedBy 
                 */
                QUrl developedBy();

                /**
                 * http://swrc.ontoware.org/ontology#develops 
                 */
                QUrl develops();

                /**
                 * http://swrc.ontoware.org/ontology#edition 
                 */
                QUrl edition();

                /**
                 * http://swrc.ontoware.org/ontology#editor 
                 */
                QUrl editor();

                /**
                 * http://swrc.ontoware.org/ontology#email 
                 */
                QUrl email();

                /**
                 * http://swrc.ontoware.org/ontology#employs 
                 */
                QUrl employs();

                /**
                 * http://swrc.ontoware.org/ontology#endDate 
                 */
                QUrl endDate();

                /**
                 * http://swrc.ontoware.org/ontology#eventTitle 
                 */
                QUrl eventTitle();

                /**
                 * http://swrc.ontoware.org/ontology#fax 
                 */
                QUrl fax();

                /**
                 * http://swrc.ontoware.org/ontology#financedBy 
                 */
                QUrl financedBy();

                /**
                 * http://swrc.ontoware.org/ontology#finances 
                 */
                QUrl finances();

                /**
                 * http://swrc.ontoware.org/ontology#firstName 
                 */
                QUrl firstName();

                /**
                 * http://swrc.ontoware.org/ontology#givenBy 
                 * 
                 * check domain. Olivier 2006-06-07 
                 * Philipp: checked. 
                 */
                QUrl givenBy();

                /**
                 * http://swrc.ontoware.org/ontology#hasPartEvent 
                 * 
                 * Example: FIFA World Cup 2006 hasPartEvent Final 
                 */
                QUrl hasPartEvent();

                /**
                 * http://swrc.ontoware.org/ontology#hasParts 
                 */
                QUrl hasParts();

                /**
                 * http://swrc.ontoware.org/ontology#head 
                 */
                QUrl head();

                /**
                 * http://swrc.ontoware.org/ontology#headOf 
                 */
                QUrl headOf();

                /**
                 * http://swrc.ontoware.org/ontology#headOfGroup 
                 * 
                 * made a subproperty of headOf. Olivier 2006-06-07 
                 */
                QUrl headOfGroup();

                /**
                 * http://swrc.ontoware.org/ontology#homepage 
                 */
                QUrl homepage();

                /**
                 * http://swrc.ontoware.org/ontology#howpublished 
                 */
                QUrl howpublished();

                /**
                 * http://swrc.ontoware.org/ontology#institution 
                 * 
                 * meaning not clear 
                 * Philipp: Hmm, I didn't create that property, therefore i don't 
                 * know the intention of it. 
                 */
                QUrl institution();

                /**
                 * http://swrc.ontoware.org/ontology#isAbout 
                 * 
                 * check domain. Olivier 2006-06-07 
                 * Philipp: checked. 
                 */
                QUrl isAbout();

                /**
                 * http://swrc.ontoware.org/ontology#isWorkedOnBy 
                 */
                QUrl isWorkedOnBy();

                /**
                 * http://swrc.ontoware.org/ontology#isbn 
                 */
                QUrl isbn();

                /**
                 * http://swrc.ontoware.org/ontology#journal 
                 */
                QUrl journal();

                /**
                 * http://swrc.ontoware.org/ontology#keywords 
                 */
                QUrl keywords();

                /**
                 * http://swrc.ontoware.org/ontology#lastName 
                 */
                QUrl lastName();

                /**
                 * http://swrc.ontoware.org/ontology#location 
                 */
                QUrl location();

                /**
                 * http://swrc.ontoware.org/ontology#member 
                 * 
                 * can an organization be a member of another organization? olivier 
                 * 2006-06-07 
                 * Philipp: Hmm could be possible. Therefore I added Organization 
                 * to the range that one can state: Organization (has)member Organization. 
                 */
                QUrl member();

                /**
                 * http://swrc.ontoware.org/ontology#memberOfPC 
                 */
                QUrl memberOfPC();

                /**
                 * http://swrc.ontoware.org/ontology#month 
                 */
                QUrl month();

                /**
                 * http://swrc.ontoware.org/ontology#name 
                 */
                QUrl name();

                /**
                 * http://swrc.ontoware.org/ontology#note 
                 */
                QUrl note();

                /**
                 * http://swrc.ontoware.org/ontology#number 
                 */
                QUrl number();

                /**
                 * http://swrc.ontoware.org/ontology#organization 
                 * 
                 * meaning not clear. Olivier 2006-06-07 
                 * Philipp: This property appears in the class definition of swrc:Document. 
                 * I think it is obsolete, since you may link a document via the dc:creator, 
                 * dc:contributor or dc:publisher property to an organization. 
                 */
                QUrl organization();

                /**
                 * http://swrc.ontoware.org/ontology#organizerOrChairOf 
                 */
                QUrl organizerOrChairOf();

                /**
                 * http://swrc.ontoware.org/ontology#outcomeDocument 
                 */
                QUrl outcomeDocument();

                /**
                 * http://swrc.ontoware.org/ontology#outcomeProduct 
                 */
                QUrl outcomeProduct();

                /**
                 * http://swrc.ontoware.org/ontology#pages 
                 */
                QUrl pages();

                /**
                 * http://swrc.ontoware.org/ontology#participant 
                 */
                QUrl participant();

                /**
                 * http://swrc.ontoware.org/ontology#phone 
                 */
                QUrl phone();

                /**
                 * http://swrc.ontoware.org/ontology#photo 
                 */
                QUrl photo();

                /**
                 * http://swrc.ontoware.org/ontology#price 
                 */
                QUrl price();

                /**
                 * http://swrc.ontoware.org/ontology#product 
                 * 
                 * meaning not clear. Olivier 2006-06-07 
                 * Philipp: Hmm, I didn't create that property, therefore i don't 
                 * know the intention of it. 
                 */
                QUrl product();

                /**
                 * http://swrc.ontoware.org/ontology#projectInfo 
                 */
                QUrl projectInfo();

                /**
                 * http://swrc.ontoware.org/ontology#publication 
                 * 
                 * check domain. Olivier 2006-06-07 
                 * Philipp: checked. 
                 */
                QUrl publication();

                /**
                 * http://swrc.ontoware.org/ontology#publisherOf 
                 * 
                 * check range. Olivier 2006-06-07 
                 * Philipp: checked. 
                 */
                QUrl publisherOf();

                /**
                 * http://swrc.ontoware.org/ontology#school 
                 * 
                 * meaning is not clear. Olivier 2006-06-07 
                 * Philipp: This property appears in the class description of 
                 * Thesis and I think should relate a Thesis to the associated university. 
                 */
                QUrl school();

                /**
                 * http://swrc.ontoware.org/ontology#series 
                 */
                QUrl series();

                /**
                 * http://swrc.ontoware.org/ontology#startDate 
                 */
                QUrl startDate();

                /**
                 * http://swrc.ontoware.org/ontology#status 
                 */
                QUrl status();

                /**
                 * http://swrc.ontoware.org/ontology#student 
                 */
                QUrl student();

                /**
                 * http://swrc.ontoware.org/ontology#studiesAt 
                 */
                QUrl studiesAt();

                /**
                 * http://swrc.ontoware.org/ontology#supervises 
                 */
                QUrl supervises();

                /**
                 * http://swrc.ontoware.org/ontology#supervisor 
                 */
                QUrl supervisor();

                /**
                 * http://swrc.ontoware.org/ontology#technicalReport 
                 * 
                 * check domain. Olivier 2006-06-07 
                 * Philipp: checked. 
                 * 
                 */
                QUrl technicalReport();

                /**
                 * http://swrc.ontoware.org/ontology#vendor 
                 */
                QUrl vendor();

                /**
                 * http://swrc.ontoware.org/ontology#volume 
                 */
                QUrl volume();

                /**
                 * http://swrc.ontoware.org/ontology#worksAtProject 
                 */
                QUrl worksAtProject();

                /**
                 * http://swrc.ontoware.org/ontology#year 
                 */
                QUrl year();
            }
        }
    }
}

#endif
