/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2008 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 */

#include "simpleresourcemodel.h"

#include <QtCore/QUrl>
#include <QtCore/QList>

#include <nepomuk/resource.h>
#include <nepomuk/class.h>

#include <KLocale>
#include <KDebug>
#include <KUrl>
#include <kcategorizedsortfilterproxymodel.h>
#include <KIcon>

#include <Soprano/Vocabulary/RDFS>


class Nepomuk::SimpleResourceModel::Private
{
public:
    QList<Nepomuk::Resource> resources;
};


Nepomuk::SimpleResourceModel::SimpleResourceModel( QObject* parent )
    : ResourceModel( parent ),
      d( new Private() )
{
}


Nepomuk::SimpleResourceModel::~SimpleResourceModel()
{
    delete d;
}


QModelIndex Nepomuk::SimpleResourceModel::indexForResource( const QUrl& uri ) const
{
    Q_ASSERT( uri.isValid() );
    // FIXME: performance
    int i = 0;
    QList<Nepomuk::Resource>::const_iterator end = d->resources.constEnd();
    for ( QList<Nepomuk::Resource>::const_iterator it = d->resources.constBegin(); it != end; ++it ) {
        if ( it->resourceUri() == uri ) {
            return index( i, 0 );
        }
        ++i;
    }

    return QModelIndex();
}


QUrl Nepomuk::SimpleResourceModel::resourceForIndex( const QModelIndex& index ) const
{
    if ( index.isValid() && index.row() < d->resources.count() ) {
        return d->resources[index.row()].resourceUri();
    }
    else {
        return QUrl();
    }
}


int Nepomuk::SimpleResourceModel::rowCount( const QModelIndex& parent ) const
{
    if ( parent.isValid() ) {
        return 0;
    }
    else {
        return d->resources.count();
    }
}


QModelIndex Nepomuk::SimpleResourceModel::index( int row, int column, const QModelIndex& parent ) const
{
    if ( row < rowCount( parent ) && column == 0 ) {
        return createIndex( row, column, 0 );
    }
    else {
        return QModelIndex();
    }
}


QVariant Nepomuk::SimpleResourceModel::data( const QModelIndex& index, int role ) const
{
    if ( index.row() < 0 || index.row() >= d->resources.count() ) {
        kDebug() << "Invalid index:" << index;
        return QVariant();
    }

    switch( role ) {
    case Qt::DisplayRole:
        return Resource( resourceForIndex( index ) ).genericLabel();

    case Qt::DecorationRole:
        return KIcon( Resource( resourceForIndex( index ) ).genericIcon() );

    case Qt::ToolTipRole:
        return KUrl( resourceForIndex( index ) ).prettyUrl();

    case ResourceUriRole:
        return resourceForIndex( index );

    case KCategorizedSortFilterProxyModel::CategorySortRole:
        // FIXME: sort files before other stuff and so on

    case KCategorizedSortFilterProxyModel::CategoryDisplayRole: {
        QUrl url = resourceForIndex( index );
        Q_ASSERT( !url.isEmpty() );
        Nepomuk::Resource res( url );
        Q_ASSERT( !res.resourceType().isEmpty() );
        Nepomuk::Types::Class c( res.resourceType() );
        QString cat = c.label();
        if ( cat.isEmpty() ) {
            cat = c.name();
        }
        if ( c.uri() == Soprano::Vocabulary::RDFS::Resource() || cat.isEmpty() ) {
            cat = i18n( "Miscellaneous" );
        }

        return cat;
    }
    }

    return ResourceModel::data( index, role );
}


void Nepomuk::SimpleResourceModel::setResources( const QList<Nepomuk::Resource>& resources )
{
    d->resources = resources;
    reset();
}

#include "simpleresourcemodel.moc"
