#include "infopanel.h"

#include <KIcon>
#include <klocale.h>
#include <nepomuk/resource.h>
#include <nepomuk/kratingwidget.h>
#include <nepomuk/kmetadatatagwidget.h>

#define INFOICONSIZE 32
#define MARGIN 5
#define ROUNDRADIUS 10
#define ICONSIZE 64
#define PANELWIDTH 600
#define PANELHEIGHT 300
#define STROKEWIDTH 1
#define STROKECOLOR 80, 80, 80
#define TEXTCOLOR 150, 150, 150
#define BACKGROUNDCOLOR 45, 45, 45, 240
#define ANIMATIONDURATION 300

InfoPanel::InfoPanel(QGraphicsItem* parent) : QObject(), QGraphicsItem(parent), m_visible(false), m_showHideAnimation(NULL)
{
	m_infoIcon = new QGraphicsPixmapItem(KIcon("help-about").pixmap(INFOICONSIZE, INFOICONSIZE), this);
	m_infoIcon->setPos(MARGIN, MARGIN);

	m_showArrow = KIcon("go-previous").pixmap(INFOICONSIZE, INFOICONSIZE);
	m_hideArrow = KIcon("go-next").pixmap(INFOICONSIZE, INFOICONSIZE);
	m_showIcon = new QGraphicsPixmapItem(m_showArrow, this);
	m_showIcon->setPos(MARGIN, (PANELHEIGHT-INFOICONSIZE)/2.);

	m_resourceIcon = new QGraphicsPixmapItem(this);
	m_resourceIcon->setPos(INFOICONSIZE+4*MARGIN, 3*MARGIN);

	QBrush textBrush;
	textBrush.setStyle(Qt::SolidPattern);
	textBrush.setColor(QColor(TEXTCOLOR));

	m_resourceTitle = new QGraphicsSimpleTextItem(this);
	QFont font = m_resourceTitle->font();
	font.setPixelSize(ICONSIZE/2);
	m_resourceTitle->setFont(font);
	m_resourceTitle->setPos(m_resourceIcon->pos() + QPointF(ICONSIZE+2*MARGIN, 0));
	m_resourceTitle->setBrush(textBrush);

	m_resourceDesc = new QGraphicsSimpleTextItem(this);
	font.setPixelSize(ICONSIZE/3);
	m_resourceDesc->setFont(font);
	m_resourceDesc->setPos(m_resourceTitle->pos() + QPointF(0, ICONSIZE/2+MARGIN));
	m_resourceDesc->setBrush(textBrush);

	QGraphicsSimpleTextItem* ratingLabel = new QGraphicsSimpleTextItem(this);
	font.setPixelSize(ICONSIZE/4);
	ratingLabel->setText(i18n("Rating"));
	ratingLabel->setFont(font);
	ratingLabel->setPos(m_resourceIcon->pos() + QPointF(0, ICONSIZE+MARGIN));
	ratingLabel->setBrush(textBrush);

	m_ratingWidget = new KRatingWidget();
	connect(m_ratingWidget, SIGNAL(ratingChanged(int)), this, SLOT(ratingChanged(int)));
	QGraphicsProxyWidget* ratingProxy = new QGraphicsProxyWidget(this);
	ratingProxy->setWidget(m_ratingWidget);
	ratingProxy->setPos(ratingLabel->pos() + QPointF(ratingLabel->boundingRect().width()+MARGIN, 0));

	QGraphicsSimpleTextItem* tagLabel = new QGraphicsSimpleTextItem(this);
	tagLabel->setText(i18n("Tags"));
	tagLabel->setFont(font);
	tagLabel->setPos(ratingLabel->pos() + QPointF(0, ratingLabel->boundingRect().height()));
	tagLabel->setBrush(textBrush);

	m_tagWidget = new Nepomuk::TagWidget();
	QGraphicsProxyWidget* tagProxy = new QGraphicsProxyWidget(this);
	tagProxy->setWidget(m_tagWidget);
	tagProxy->setPos(tagLabel->pos() + QPointF(tagLabel->boundingRect().width() + MARGIN, 0));

	m_pen.setColor(QColor(STROKECOLOR));
	m_pen.setWidthF(STROKEWIDTH);

	QLinearGradient backgroundGradient(0, 0, PANELWIDTH, 0);
	backgroundGradient.setColorAt(0, QColor(STROKECOLOR));
	backgroundGradient.setColorAt((INFOICONSIZE+2.*MARGIN)/PANELWIDTH, QColor(BACKGROUNDCOLOR));
	backgroundGradient.setColorAt(1, QColor(BACKGROUNDCOLOR));
	m_brush = QBrush(backgroundGradient);
}

void InfoPanel::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
{
	painter->save();

	painter->setPen(m_pen);
	painter->setBrush(m_brush);

	painter->drawRoundedRect(0, 0, PANELWIDTH+ROUNDRADIUS, PANELHEIGHT, ROUNDRADIUS, ROUNDRADIUS);

	painter->restore();
}

QRectF InfoPanel::boundingRect() const
{
	return QRectF(0, 0, PANELWIDTH, PANELHEIGHT);
}

void InfoPanel::showPanel()
{
	if(m_visible)
		return;
	m_visible = true;
	m_showIcon->setPixmap(m_hideArrow);
	m_initialPos = pos();

	if(m_showHideAnimation)
		m_showHideAnimation->deleteLater();

	m_showHideAnimation = new QGraphicsItemAnimation(this);
	QTimeLine* timeline = new QTimeLine(ANIMATIONDURATION, m_showHideAnimation);
	timeline->setFrameRange(0, 1);
	m_showHideAnimation->setItem(this);
	m_showHideAnimation->setTimeLine(timeline);
	m_showHideAnimation->setPosAt(0, pos());
	m_showHideAnimation->setPosAt(1, pos()-QPointF(PANELWIDTH-INFOICONSIZE-2*MARGIN, 0));
	timeline->start();
}

void InfoPanel::hidePanel()
{
	if(!m_visible)
		return;
	m_visible = false;
	m_showIcon->setPixmap(m_showArrow);

	if(m_showHideAnimation)
		m_showHideAnimation->deleteLater();

	m_showHideAnimation = new QGraphicsItemAnimation(this);
	QTimeLine* timeline = new QTimeLine(ANIMATIONDURATION, m_showHideAnimation);
	timeline->setFrameRange(0, 1);
	m_showHideAnimation->setItem(this);
	m_showHideAnimation->setTimeLine(timeline);
	m_showHideAnimation->setPosAt(0, pos());
	m_showHideAnimation->setPosAt(1, pos()+QPointF(PANELWIDTH-INFOICONSIZE-2*MARGIN, 0));
	timeline->start();
}

bool InfoPanel::visible() const
{
	return m_visible;
}

void InfoPanel::setResource(const Nepomuk::AnnotationResource& resource)
{
	m_resource = resource;

	QPixmap icon;
	if(m_resource.icon().type() == QVariant::String) {
		icon = KIcon(m_resource.icon().toString()).pixmap(ICONSIZE, ICONSIZE);
	} else {
		icon = QPixmap::fromImage(m_resource.icon().value<QImage>().scaled(ICONSIZE, ICONSIZE, Qt::KeepAspectRatio));
	}

	m_resourceIcon->setPixmap(icon);

	m_resourceTitle->setText(resource.label());
	m_resourceDesc->setText(resource.description());

	Nepomuk::Resource res(m_resource.resource().resourceUri());
	m_ratingWidget->setRating(res.rating());

	m_tagWidget->setTaggedResource(res);
}

float InfoPanel::tabWidth()
{
	return INFOICONSIZE+2*MARGIN;
}

void InfoPanel::ratingChanged(int rating)
{
	Nepomuk::Resource res(m_resource.resource().resourceUri());
	res.setRating(rating);
}

void InfoPanel::mousePressEvent(QGraphicsSceneMouseEvent* event)
{
	event->accept();
}

void InfoPanel::mouseReleaseEvent(QGraphicsSceneMouseEvent* event)
{
	if(event->button() != Qt::LeftButton)
		return;

	// Click on the tab
	if(event->pos().x() < MARGIN+INFOICONSIZE) {
		if(m_visible) {
			hidePanel();
		} else {
			showPanel();
		}
	}
}
#include "infopanel.moc"
