#include "planet.h"

#include <Nepomuk/Resource>
#include <kicon.h>
#include <kglobalsettings.h>
#include <KRun>

#define ICONSIZE 64. // Icons are fitted into a 64x64 square
#define SQRT2 1.4
#define PI 3.14
#define RADTODEG (180/PI)
#define MARGIN 10. // Minimum space between two items
#define PLANETSTROKEWIDTH 7.
#define PLANETRADIUS (ICONSIZE/SQRT2 + MARGIN)

#define MOVEANIMATIONDURATION 500 // milliseconds
#define SHOWHIDEANIMATIONDURATION 500 // milliseconds

#define PLANETSTROKECOLOR 80, 80, 80
#define PLANETBACKGROUND 45, 45, 45

Planet::Planet(const Nepomuk::AnnotationResource& resource) : QObject(), QGraphicsItem(), m_resource(resource), m_showActions(false), m_moveAnimation(0), m_alpha(1), m_visible(true), m_active(false)
{
	m_planetPen.setColor(QColor(PLANETSTROKECOLOR));
	m_planetPen.setWidthF(PLANETSTROKEWIDTH);

	m_planetBrush.setStyle(Qt::SolidPattern);
	m_planetBrush.setColor(QColor(PLANETBACKGROUND));

	// Cache the icon
	if(m_resource.icon().type() == QVariant::String) {
		m_icon = KIcon(m_resource.icon().toString()).pixmap(ICONSIZE, ICONSIZE);
	} else {
		m_icon = QPixmap::fromImage(m_resource.icon().value<QImage>().scaled(ICONSIZE, ICONSIZE, Qt::KeepAspectRatio));
	}

	m_showHideTimeline = new QTimeLine(SHOWHIDEANIMATIONDURATION, this);
	m_showHideTimeline->setFrameRange(0, 100);
	connect(m_showHideTimeline, SIGNAL(frameChanged(int)), this, SLOT(setAlpha(int)));
}

Planet::~Planet()
{
}

void Planet::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
{
	painter->save();
	
	painter->setOpacity(m_alpha);

	QPen pen = m_planetPen;
	QColor color = pen.color();
	if(m_active)
		color.setHsvF(color.hueF(), color.saturationF(), qMin(color.valueF()+0.5, 1.));
	pen.setColor(color);
	painter->setPen(pen);

	painter->setBrush(m_planetBrush);
	painter->setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing);

	// Paint the background
	painter->drawEllipse(-PLANETRADIUS, -PLANETRADIUS, 2*PLANETRADIUS, 2*PLANETRADIUS);


	// Draw the icon
	painter->drawPixmap(-ICONSIZE/2, -ICONSIZE/2, m_icon);

	// Draw the label (I hope you like magic numbers)
	QFont font = KGlobalSettings::generalFont();
	font.setPixelSize(ICONSIZE/3);
	painter->setFont(font);

	const int textWidth = 2*(PLANETSTROKEWIDTH + 2*PLANETRADIUS);
	const int textX = -textWidth/2;
	const int textY = PLANETRADIUS;
	QString text = painter->fontMetrics().elidedText(m_resource.label(), Qt::ElideRight, textWidth);
	painter->drawText(textX, textY, textWidth, painter->fontMetrics().height(), Qt::AlignHCenter | Qt::AlignBottom, text);

	painter->restore();
}

QRectF Planet::boundingRect() const
{
	const float width = 2*(PLANETSTROKEWIDTH + 2*PLANETRADIUS);
	const float height = 2*PLANETRADIUS+PLANETSTROKEWIDTH + ICONSIZE/3;
	return QRectF(-width/1.5, -height/1.5, 1.5*width, 1.5*height);
}

void Planet::setPos(const QPointF& newPos)
{
	// If there's already an animation happening, delete it
	if(m_moveAnimation) {
		// The timeline is a children of the animation, so it'll get deleted too
		m_moveAnimation->deleteLater();
	}

	m_moveAnimation = new QGraphicsItemAnimation(this);
	QTimeLine* timeline = new QTimeLine(MOVEANIMATIONDURATION, m_moveAnimation);
	timeline->setFrameRange(0, 1);
	m_moveAnimation->setItem(this);
	m_moveAnimation->setTimeLine(timeline);
	m_moveAnimation->setPosAt(0, pos());
	m_moveAnimation->setPosAt(1, newPos);
	timeline->start();
}

void Planet::setPos(qreal x, qreal y)
{
	setPos(QPointF(x, y));
}

void Planet::setPosImmediate(const QPointF& newPos)
{
	QGraphicsItem::setPos(newPos);
}

void Planet::setPosImmediate(qreal x, qreal y)
{
	QGraphicsItem::setPos(x, y);
}

bool Planet::visible() const
{
	return m_visible;
}

void Planet::setActive(bool active)
{
	m_active = active;
}

void Planet::setActions(const QList<KSharedPtr<KService> >& actions)
{
	ActionButton* currentButton;
	foreach(const KSharedPtr<KService>& action, actions) {
		currentButton = new ActionButton(KIcon(action->icon()), action->comment(), this);
		currentButton->setVisible(m_showActions);
		m_execCommands.insert(currentButton, action->exec());
		connect(currentButton, SIGNAL(clicked()), this, SLOT(actionButtonClicked()));
		connect(currentButton, SIGNAL(mouseOver()), this, SLOT(action_mouseOver()));
		connect(currentButton, SIGNAL(mouseOut()), this, SLOT(action_mouseOut()));
	}

	placeActionButtons();
}

void Planet::showActions(bool show)
{
	m_showActions = show;
	foreach(ActionButton* button, m_execCommands.keys()) {
		button->setVisible(m_showActions);
	}
}

float Planet::radius()
{
	return PLANETRADIUS + PLANETSTROKEWIDTH/2;
}

float Planet::margin()
{
	return MARGIN;
}

const Nepomuk::AnnotationResource& Planet::resource()
{
	return m_resource;
}

void Planet::show()
{
	if(m_visible) return;
	m_visible = true;
	emit visibilityChanged();
	m_showHideTimeline->stop();
	m_showHideTimeline->setDirection(QTimeLine::Forward);
	m_showHideTimeline->start();
}

void Planet::hide()
{
	if(!m_visible) return;
	m_visible = false;
	emit visibilityChanged();
	m_showHideTimeline->stop();
	m_showHideTimeline->setDirection(QTimeLine::Backward);
	m_showHideTimeline->start();
}

void Planet::placeActionButtons()
{
	if(m_execCommands.empty())
		return;

	// TODO : My math doesn't work... But I'm almost sure they're correct ! The action buttons are not well placed.
	QList<ActionButton*> actionButtons = m_execCommands.keys();
	int n_actions = actionButtons.size();
	float radius = Planet::radius() + ActionButton::radius() + 2*Planet::margin();
	float angleIncrement = (n_actions*ActionButton::radius()+(n_actions-1)*ActionButton::margin()) / (Planet::radius()+ActionButton::radius()+ActionButton::margin());
	float initialAngle = PI/2 - (n_actions - 1)/2*angleIncrement;

	for(int i = 0 ; i < n_actions ; ++i) {
		QLineF position = QLineF::fromPolar(radius, RADTODEG*(i * angleIncrement + initialAngle));
		actionButtons.at(i)->setPos(position.p2());
	}
}

void Planet::actionButtonClicked()
{
	ActionButton* button = qobject_cast<ActionButton*>(sender());
	if(!button)
		return;

	qDebug() << "launching" << m_execCommands[button].replace("%u", m_resource.resource().uri()).replace("%t", m_resource.label());
	KRun::runCommand(m_execCommands[button].replace("%u", m_resource.resource().uri()).replace("%t", m_resource.label()), scene()->views().first());
}

void Planet::setAlpha(int value)
{
	m_alpha = value/100.;
	update();
}

void Planet::action_mouseOver()
{
	ActionButton* button = qobject_cast<ActionButton*>(sender());
	if(!button)
		return;

	emit actionText(button->label());
}

void Planet::action_mouseOut()
{
	emit actionText(QString());
}

QVariant Planet::itemChange(GraphicsItemChange change, const QVariant& value)
{
	if(change == ItemPositionChange && scene()) {
		emit moved(value.toPointF());
	}

	return QGraphicsItem::itemChange(change, value);
}

void Planet::mousePressEvent(QGraphicsSceneMouseEvent* event)
{
	event->accept();
}

void Planet::mouseReleaseEvent(QGraphicsSceneMouseEvent* event)
{
	emit clicked();
}

#include "planet.moc"
