#include "resourceinspector.h"

#include "planet.h"
#include "infopanel.h"
#include "annotationplugin.h"
#include "annotationresult.h"
#include "annotationpluginfactory.h"
#include <Nepomuk/Resource>
#include <soprano/soprano.h>
#include <KServiceTypeTrader>
#include <KIcon>
#include <KLineEdit>

#define WINDOWWIDTH 800
#define WINDOWHEIGHT 600

#define FILTERGREEN 57, 188, 43
#define TEXTCOLOR 80, 80, 80
#define LINKCOLOR 120, 120, 120
#define LINKSTROKEWIDTH 1

#define PI 3.14
#define SQRT2 1.4

#define ZINDEX_LINK 0
#define ZINDEX_PLANET 1
#define ZINDEX_GUI 2

#define RDF_RESOURCE "http://www.w3.org/2000/01/rdf-schema#Resource"

ResourceInspector::ResourceInspector(QWidget* parent) : QGraphicsView(parent), m_centralPlanet(0), m_currentFilterPlugin(0)
{
	/*
	 * Scene layers :
	 * Links : 0
	 * Planets : 1
	 * GUI items : 2
	 */
	m_scene = new QGraphicsScene(this);
	setScene(m_scene);

	//setWindowFlags(Qt::CustomizeWindowHint | Qt::FramelessWindowHint); // Will be used later to look like KRunner or such
	
	// Use a black background
	setBackgroundBrush(Qt::black);

	// Disable scrolling
	setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

	// Antialias the whole scene
	setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing | QPainter::SmoothPixmapTransform);

	// Pen used to draw the links between the main resource and the satellites
	m_linkPen.setColor(QColor(LINKCOLOR));
	m_linkPen.setWidthF(LINKSTROKEWIDTH);

	resize(WINDOWWIDTH, WINDOWHEIGHT);
	setSceneRect(0, 0, WINDOWWIDTH, WINDOWHEIGHT);
	m_sceneCenter.setX(WINDOWWIDTH/2.);
	m_sceneCenter.setY(WINDOWHEIGHT/2.);
	
	////// Build the action cache (associate rdf types with possible actions)
	KService::List services = KServiceTypeTrader::self()->query("Nepomuk/ResourceRunner");
	foreach(const KSharedPtr<KService>& service, services) {
		qDebug() << "loading runner for resource type" << service->property("X-KDE-NepomukResourceType", QVariant::String).toString();
		m_actions.insertMulti(service->property("X-KDE-NepomukResourceType", QVariant::String).toString(), service);
	}

	QShortcut* closeShortcut = new QShortcut(QKeySequence(Qt::Key_Escape), this, SLOT(escapeKey_cb()));

	////// Create the "show all" filter ActionButton
	m_showAllButton = new ActionButton(KIcon("rating"), i18n("Show all"));
	m_showAllButton->setColor(QColor(FILTERGREEN));
	m_showAllButton->setVisible(false);
	m_showAllButton->setZValue(ZINDEX_GUI);
	connect(m_showAllButton, SIGNAL(mouseOver()), this, SLOT(filterButton_hover()));
	connect(m_showAllButton, SIGNAL(mouseOut()), this, SLOT(resetStatusMessage()));
	connect(m_showAllButton, SIGNAL(clicked()), this, SLOT(setFilterPlugin()));
	m_scene->addItem(m_showAllButton);

	////// Create the search field
	KLineEdit* searchField = new KLineEdit;
	m_searchFieldProxy = m_scene->addWidget(searchField);
	m_searchFieldProxy->setFlag(QGraphicsItem::ItemIgnoresTransformations);
	m_searchFieldProxy->setZValue(ZINDEX_GUI);
	searchField->setClearButtonShown(true);
	searchField->setClickMessage(i18n("Search"));
	// Make the search field background transparent
	QPalette palette(searchField->palette());
	palette.setColor(QPalette::Base, Qt::black);
	palette.setColor(QPalette::Text, QColor(TEXTCOLOR));
	searchField->setPalette(palette);
	connect(searchField, SIGNAL(textChanged(const QString&)), this, SLOT(filterPlanets()));
	m_searchFieldProxy->grabKeyboard();
	m_searchFieldProxy->setPos(4*ActionButton::radius()+ActionButton::margin(), ActionButton::margin());

	////// Set up the status message at the top of the window
	m_textBrush.setStyle(Qt::SolidPattern);
	m_textBrush.setColor(QColor(TEXTCOLOR));
	m_statusMessage = new QGraphicsSimpleTextItem();
	m_statusMessage->setFlag(QGraphicsItem::ItemIgnoresTransformations);
	m_statusMessage->setZValue(ZINDEX_GUI);
	m_statusMessage->setBrush(m_textBrush);
	QFont font = m_statusMessage->font();
	font.setPixelSize(ActionButton::radius());
	m_statusMessage->setFont(font);
	m_scene->addItem(m_statusMessage);
	m_statusMessage->setPos(m_searchFieldProxy->pos()+QPointF(m_searchFieldProxy->boundingRect().width()+ActionButton::radius(), 0));

	////// Set up the info panel
	m_infoPanel = new InfoPanel();
	m_scene->addItem(m_infoPanel);
	m_infoPanel->setZValue(ZINDEX_GUI);
}

ResourceInspector::~ResourceInspector()
{
}

void ResourceInspector::setResource(const QUrl& resourceUri)
{
	// First clean the scene
	foreach(QGraphicsItem* i, m_satellites) {
		if(i != m_centralPlanet) { // Can happen in the case where satellite_clicked was called before
			m_scene->removeItem(i);
			delete i;
		}
	}
	m_satellites.clear();

	foreach(QGraphicsItem* i, m_satelliteLinks) {
		m_scene->removeItem(i);
		delete i;
	}
	m_satelliteLinks.clear();

	foreach(QGraphicsItem* i, m_filterPlugins.keys()) {
		if(i == m_showAllButton)
			continue;

		m_scene->removeItem(i);
		delete i;
	}
	m_filterPlugins.clear();
	m_filterPlugins.insert(m_showAllButton, NULL);

	Nepomuk::Resource r(resourceUri);

	if(!r.exists())
		return;

	// Note : we could also use Nepomuk::Resource::types to try harder to get a plugin
	QList<Nepomuk::AnnotationPlugin*> plugins = Nepomuk::AnnotationPluginFactory::instance()->getPluginsForType(QUrl(r.type()));
	if(plugins.empty()) {
		qDebug() << "Could not find any plugin for resource type" << r.type();
		return;
	}

	// What should we do if we have several plugins ?
	Nepomuk::AnnotationPlugin* plugin = plugins.first();
	m_currentUri = resourceUri;

	Nepomuk::AnnotationResult* result = plugin->getResource(m_currentUri);
	connect(result, SIGNAL(ready()), this, SLOT(result_cb()));
	result->emitStatus();

	// Fetch also the linked resources
	QList<Nepomuk::AnnotationPlugin*> allPlugins = Nepomuk::AnnotationPluginFactory::instance()->getAllPlugins();
	foreach(Nepomuk::AnnotationPlugin* p, allPlugins) {
		result = p->getResourcesRelatedTo(m_currentUri);
		connect(result, SIGNAL(ready()), this, SLOT(relatedResult_cb()));
		result->emitStatus();
	}
}

void ResourceInspector::showFilterButtons(bool show)
{
	if(show) {
		QPointF currentPos = sceneRect().topLeft();
		currentPos += QPointF(ActionButton::margin()+ActionButton::radius(), ActionButton::margin()+ActionButton::radius());
		QPointF shift(0, 2*ActionButton::radius() + ActionButton::margin());

		foreach(ActionButton* b, m_filterPlugins.keys()) {
			b->setPos(currentPos);
			b->setVisible(true);
			currentPos += shift;
		}
	}

	foreach(ActionButton* b, m_filterPlugins.keys()) {
		b->setVisible(show);
	}

}

void ResourceInspector::result_cb()
{
	Nepomuk::AnnotationResult* result = qobject_cast<Nepomuk::AnnotationResult*>(sender());
	if(!result)
		return;

	QList<Nepomuk::AnnotationResource> resources = result->resources();
	delete result;

	if(resources.empty()) {
		qDebug() << "cannot find the resource ???";
		return;
	}

	if(!m_centralPlanet) {
		m_centralPlanet = new Planet(resources.first());
		m_centralPlanet->setActions(m_actions.values(m_centralPlanet->resource().resource().type()) + m_actions.values(RDF_RESOURCE));
		m_scene->addItem(m_centralPlanet);
	}
	m_centralPlanet->setPos(m_sceneCenter);
	m_centralPlanet->setActive(true);
	m_centralPlanet->setZValue(ZINDEX_PLANET);
	m_centralPlanet->showActions(true);
	connect(m_centralPlanet, SIGNAL(actionText(const QString&)), this, SLOT(actionButton_hover(const QString&)));

	m_infoPanel->setResource(m_centralPlanet->resource());
}

void ResourceInspector::relatedResult_cb()
{
	Nepomuk::AnnotationResult* result = qobject_cast<Nepomuk::AnnotationResult*>(sender());
	Nepomuk::AnnotationPlugin* plugin = result->parentPlugin();

	if(!result)
		return;

	QList<Nepomuk::AnnotationResource> resources = result->resources();
	delete result;

	if(resources.empty()) {
		return;
	}

	// Create the filter ActionButton
	ActionButton* button = new ActionButton(KIcon(plugin->iconName()), plugin->name());
	button->setZValue(ZINDEX_GUI);
	button->setColor(QColor(FILTERGREEN));
	button->setVisible(false);
	connect(button, SIGNAL(mouseOver()), this, SLOT(filterButton_hover()));
	connect(button, SIGNAL(mouseOut()), this, SLOT(resetStatusMessage()));
	connect(button, SIGNAL(clicked()), this, SLOT(setFilterPlugin()));

	m_scene->addItem(button);
	m_filterPlugins[button] = plugin;

	// Create the planets from the results
	Planet* currentPlanet;
	QGraphicsLineItem* link;
	foreach(const Nepomuk::AnnotationResource& r, resources) {
		currentPlanet = new Planet(r);
		currentPlanet->setActions(m_actions.values(r.resource().type()) + m_actions.values(RDF_RESOURCE));
		m_satellites.append(currentPlanet);
		m_scene->addItem(currentPlanet);
		connect(currentPlanet, SIGNAL(moved(const QPointF&)), this, SLOT(updateLink(const QPointF&)));
		currentPlanet->setZValue(ZINDEX_PLANET);

		// Add the link between the central resource and the main resource
		link = new QGraphicsLineItem(0, 0, currentPlanet->x(), currentPlanet->y());
		link->setZValue(ZINDEX_LINK);
		link->setPen(m_linkPen);
		m_scene->addItem(link);
		m_satelliteLinks.insert(currentPlanet, link);
		connect(currentPlanet, SIGNAL(moved(const QPointF&)), this, SLOT(updateLink(const QPointF&)));
		connect(currentPlanet, SIGNAL(clicked()), this, SLOT(satellite_clicked()));
		connect(currentPlanet, SIGNAL(visibilityChanged()), this, SLOT(updateLinkVisibility()));

		currentPlanet->setPosImmediate(m_sceneCenter);
	}

	placeSatellites();
	showFilterButtons();
}

float ResourceInspector::systemRadius() const
{
	// Scientific version : more satellites -> wider radius, not useful in real life (tm)
	//const int n_satellites = m_satellites.size();
	//return 1/(2*PI) * (2 * n_satellites * Planet::radius() + (n_satellites - 1) * Planet::margin()) + 2*Planet::radius();

	// Working version (yeah, magic numbers again !)
	return 4*(Planet::radius());
}

void ResourceInspector::placeSatellites()
{
	if(m_satellites.empty())
		return;

	QList<Planet*> visibleSatellites;
	foreach(Planet* p, m_satellites) {
		if(p->visible())
			visibleSatellites.append(p);
	}
	if(visibleSatellites.empty())
		return;

	const int n_satellites = visibleSatellites.size();
	const float angleIncrement = 360/n_satellites;
	const float radius = systemRadius();

	for(int i = 0 ; i < n_satellites ; ++i) {
		QLineF position = QLineF::fromPolar(radius, i * angleIncrement);
		visibleSatellites.at(i)->setPos(position.p2()+m_sceneCenter);
	}
}

void ResourceInspector::updateLink(const QPointF& newPos)
{
	Planet* p = qobject_cast<Planet*>(sender());
	if(!p)
		return;

	QGraphicsLineItem* line = m_satelliteLinks.value(p);
	if(p == m_centralPlanet) // Can happen when the satellite is moved to center, after becoming m_centralPlanet
		return;

	line->setLine(m_sceneCenter.x(), m_sceneCenter.y(), newPos.x(), newPos.y());
}

void ResourceInspector::updateLinkVisibility()
{
	Planet* p = qobject_cast<Planet*>(sender());
	if(!p)
		return;

	QGraphicsLineItem* line = m_satelliteLinks.value(p);

	if(p->visible())
		line->show();
	else
		line->hide();
}

void ResourceInspector::satellite_clicked()
{
	Planet* p = qobject_cast<Planet*>(sender());
	if(!p)
		return;

	if(p == m_centralPlanet)
		return;

	qDebug() << "now watching uri" << p->resource().resource().resourceUri();

	m_scene->removeItem(m_centralPlanet);
	delete m_centralPlanet;
	m_centralPlanet = p;

	showFilterButtons(false);
	setResource(p->resource().resource().resourceUri());
}

void ResourceInspector::actionButton_hover(const QString& message)
{
	m_statusMessage->setText(message);
}

void ResourceInspector::filterButton_hover()
{
	ActionButton* button = qobject_cast<ActionButton*>(sender());
	if(!button)
		return;

	if(button == m_showAllButton) {
		m_statusMessage->setText(m_showAllButton->label());
	} else {
		m_statusMessage->setText(i18n("Show only items of type \"%1\"", button->label()));
	}
}

void ResourceInspector::resetStatusMessage()
{
	m_statusMessage->setText("");
}

void ResourceInspector::setFilterPlugin()
{
	ActionButton* button = qobject_cast<ActionButton*>(sender());
	if(!button)
		return;

	// Handle the special case "Show all"
	if(button == m_showAllButton) {
		m_currentFilterPlugin = NULL;
	} else {
		m_currentFilterPlugin = m_filterPlugins[button];
	}

	filterPlanets();
}

void ResourceInspector::filterPlanets()
{
	QString textFilter = static_cast<KLineEdit*>(m_searchFieldProxy->widget())->text();
	foreach(Planet* p, m_satellites) {
		bool show = false;
		if(p->resource().parent() == m_currentFilterPlugin || m_currentFilterPlugin == NULL) { // Group filtering
			if(p->resource().label().contains(textFilter, Qt::CaseInsensitive))
				show = true;
		}

		if(show)
			p->show();
		else
			p->hide();
	}

	placeSatellites();
}

void ResourceInspector::escapeKey_cb()
{
	if(m_infoPanel->visible()) {
		m_infoPanel->hidePanel();
		return;
	}

	KLineEdit* searchField = static_cast<KLineEdit*>(m_searchFieldProxy->widget());
	if(!searchField->text().isEmpty()) {
		searchField->clear();
		return;
	}

	close();
}

void ResourceInspector::resizeEvent(QResizeEvent* event)
{
	int w = event->size().width();
	int h = event->size().height();
	setSceneRect(0, 0, w, h);
	m_sceneCenter.setX(w/2.);
	m_sceneCenter.setY(h/2.);

	if(m_centralPlanet)
		m_centralPlanet->setPos(m_sceneCenter);

	placeSatellites();

	if(m_infoPanel->visible()) {
		m_infoPanel->setPos(w-m_infoPanel->boundingRect().width(), (h-m_infoPanel->boundingRect().height())/2);
	} else {
		m_infoPanel->setPos(w-m_infoPanel->tabWidth(), (h-m_infoPanel->boundingRect().height())/2);
	}
}

void ResourceInspector::wheelEvent(QWheelEvent* event)
{
	// |Delta| = 120 for a wheel increment
	qreal factor = 1. +  event->delta()/720.;
	scale(factor, factor);
	event->accept();
}

#include "resourceinspector.moc"
